(**************************************************************************
*                                                                         *
*  package  FORMEL                             Version  1.02              *
*                                                                         *
***************************************************************************
     *                                                                    *
     *  J. Desteffani IA85                     17.02.1987 - 21.12.1987    *
     *                                                                    *
     *  p u r p o s e :                                                   *
     *     ... INITIALISIERUNG ...                                        *
     *     Diese Procedure muss immer am Anfang eines Programmes das      *
     *     dieses Package verwendet aufgerufen werden.                    *
     *                                                                    *
     *     ... TRANSLATE_FORMEL ...                                       *
     *     Formt die in einen String eingelesene Formel in einen          *
     *       Pointerbaum um.                                              *
     *     Eine unkorrekte Formel bewirkt die Rueckgabe der Fehler -      *
     *       position (error_pos) in dem Formelstring.                    *
     *     Die Formel wird in dieser Procedure bearbeitet und kommt im    *
     *       String selber veraendert zurueck. D.h. die Spaces werden     *
     *       herausgenommen und Grossbuchstaben werden in Kleinbuchstaben *
     *       verwandelt.                                                  *
     *     Die Formel wird, soweit moeglich, optimiert. D.h. alle Be -    *
     *       rechnungen die kein x enthalten werden ausgefuehrt.          *
     *                                                                    *
     *     ... CALC ...                                                   *
     *     Berechnet die spezifizierte (root) Formel fuer einen be -      *
     *       x-Wert.                                                      *
     *     Error_nr = 0 heisst, dass kein Fehler aufgetreten ist.         *
     *       Je nach Fehlerart wird ein anderer Fehlercode ausgegeben.    *
     *                                                                    *
     *     ... GET_Y_MARGINS ...                                          *
     *     Berechnet den maximalen (bzw. minimalen Wert) der bei der      *
     *       spezifizierten (root) Formel im Bereich x_min bis x_max      *
     *       erreicht wird.                                               *
     *     In diesem Bereich werden <xmax> Punkte berechnet.              *
     *                                                                    *
     *     ... TAB_CALC ...                                               *
     *     Diese Procedure berechnet eine Tabelle im Bereich x_min bis    *
     *       x_max mit der spezifizierten Funktion (root).                *
     *     Es werden <xmax> Punkte berechnet. (Im Array 000..<xmax>). Der *
     *       y-Array enthaelt nach der Berechnung Zahlen im Bereich von   *
     *       000...<ymax>.                                                *
     *     Ist das Formelresultat ausserhalb des y-Bereiches, wird bei    *
     *       Unterschreitung -1, bei Ueberschreitung <ymax> + 1           *
     *       abgespeichert. Bei einem voellig unbrauchbaren Ergebnis      *
     *       wird -2 ausgegeben.                                          *
     *                                                                    *
     *  e n t r i e s :                                                   *
     *                                                                    *
     *  PROCEDURE Initialisierung;                                        *
     *                                                                    *
     *  PROCEDURE Translate_Formel (VAR formula      :T_string80;         *
     *                              VAR root         :T_formel_pointer;   *
     *                              VAR error_pos    :INTEGER);           *
     *                                                                    *
     *  PROCEDURE Calc             (    root         :T_formel_pointer;   *
     *                                  x            :REAL;               *
     *                              VAR y            :REAL;               *
     *                              VAR error_nr     :INTEGER);           *
     *                                                                    *
     *  PROCEDURE Get_y_Margins    (    root         :T_formel_pointer;   *
     *                                  x_min, x_max :REAL;               *
     *                              VAR y_min, y_max :REAL);              *
     *                                                                    *
     *  PROCEDURE Tab_Calc         (    x_min, x_max :REAL;               *
     *                                  y_min, y_max :REAL;               *
     *                              VAR Tabelle      :T_Tabelle);         *
     *                                                                    *
     **********************************************************************)

   CONST
      max_anz_befehle = 99;
      pi = 3.141592654;

      C_error_w   = '';        C_error   =  0;
      C_var_w     = '';        C_var     =  1;   (* explizite Variable *)
      C_x_w       = 'x';       C_x       =  2;   (* Variable x *)
      C_pi_w      = 'pi';      C_pi      =  5;   (* Zahl Pi *)
      C_random_w  = 'rand';    C_random  =  9;   (* Zufallszahl in (0,1) *)
      C_kauf_w    = '(';       C_kauf    = 10;   (* Klammer auf *)
      C_kzu_w     = ')';       C_kzu     = 11;   (* Klammer zu *)
      C_plus_w    = '+';       C_plus    = 20;
      C_minus_w   = '-';       C_minus   = 25;
      C_mal_w     = '*';       C_mal     = 30;
      C_geteilt_w = '/';       C_geteilt = 35;
      C_hoch_w    = '^';       C_hoch    = 40;
      C_sin_w     = 'sin';     C_sin     = 50;
      C_cos_w     = 'cos';     C_cos     = 51;
      C_tan_w     = 'tan';     C_tan     = 52;
      C_sinh_w    = 'sinh';    C_sinh    = 53;
      C_cosh_w    = 'cosh';    C_cosh    = 54;
      C_tanh_w    = 'tanh';    C_tanh    = 55;
      C_asin_w    = 'asin';    C_asin    = 61;
      C_acos_w    = 'acos';    C_acos    = 62;
      C_atan_w    = 'atan';    C_atan    = 63;
      C_asinh_w   = 'asinh';   C_asinh   = 64;
      C_acosh_w   = 'acosh';   C_acosh   = 65;
      C_atanh_w   = 'atanh';   C_atanh   = 66;
      C_ln_w      = 'ln';      C_ln      = 70;   (* log mit Basis e  *)
      C_log_w     = 'log';     C_log     = 71;   (* log mit Basis 10 *)
      C_exp_w     = 'exp';     C_exp     = 75;   (* Inverse von ln *)
      C_sqr_w     = 'sqr';     C_sqr     = 80;   (* Quadrat *)
      C_sqrt_w    = 'sqrt';    C_sqrt    = 81;   (* Quadratwurzel *)
      C_abs_w     = 'abs';     C_abs     = 90;   (* Absolutbetrag *)
      C_frac_w    = 'frac';    C_frac    = 91;   (* Bruchteil *)
      C_int_w     = 'int';     C_int     = 92;   (* Vorkommastellen *)



   VAR
      code   :ARRAY[0..max_anz_befehle] OF String[8];


(*------------------------------------------------------------------------*)


  (************************************************************************
   *                                                                      *
   *  PROCEDURE Initialisierung                                           *
   *                                                                      *
   ************************************************************************
        *                                                                 *
        *  Diese Procedure muss immer am Anfang eines Programmes das      *
        *  dieses Package verwendet aufgerufen werden.                    *
        *                                                                 *
        *******************************************************************)


   PROCEDURE Initialisierung;

      VAR
         index  :INTEGER;

   BEGIN
      FOR index:= 0 TO max_anz_befehle DO BEGIN
         code[index]:= '';
      END;
      code[C_error  ] := C_error_w;
      code[C_var    ] := C_var_w;
      code[C_x      ] := C_x_w;
      code[C_pi     ] := C_pi_w;
      code[C_random ] := C_random_w;
      code[C_kauf   ] := C_kauf_w;
      code[C_kzu    ] := C_kzu_w;
      code[C_plus   ] := C_plus_w;
      code[C_minus  ] := C_minus_w;
      code[C_mal    ] := C_mal_w;
      code[C_geteilt] := C_geteilt_w;
      code[C_hoch   ] := C_hoch_w;
      code[C_sin    ] := C_sin_w;
      code[C_cos    ] := C_cos_w;
      code[C_tan    ] := C_tan_w;
      code[C_sinh   ] := C_sinh_w;
      code[C_cosh   ] := C_cosh_w;
      code[C_tanh   ] := C_tanh_w;
      code[C_asin   ] := C_asin_w;
      code[C_acos   ] := C_acos_w;
      code[C_atan   ] := C_atan_w;
      code[C_asinh  ] := C_asinh_w;
      code[C_acosh  ] := C_acosh_w;
      code[C_atanh  ] := C_atanh_w;
      code[C_ln     ] := C_ln_w;
      code[C_log    ] := C_log_w;
      code[C_exp    ] := C_exp_w;
      code[C_sqr    ] := C_sqr_w;
      code[C_sqrt   ] := C_sqrt_w;
      code[C_abs    ] := C_abs_w;
      code[C_frac   ] := C_frac_w;
      code[C_int    ] := C_int_w;
   END;

(*------------------------------------------------------------------------*)


  (************************************************************************
   *                                                                      *
   *  PROCEDURE Calc                                                      *
   *                                                                      *
   ************************************************************************
        *                                                                 *
        *     Berechnet die spezifizierte (root) Formel fuer einen be -   *
        *       x-Wert.                                                   *
        *     Error_nr = 0 heisst, dass kein Fehler aufgetreten ist.      *
        *                                                                 *
        *     Beispiel:                                                   *
        *        Error_Nr = 30 >>> Ergebnis ist voellig unbrauchbar       *
        *                    ^     ((-3)^(1.25) zum Beispiel)             *
        *        Error_Nr = 11 >>> Ergebnis ist + unendlich.              *
        *                    ^     (1/0 zum Beispiel)                     *
        *        Error_Nr = 12 >>> Ergebnis ist - unendlich.              *
        *                          (-3/0 zum Beispiel)                    *
        *                                                                 *
        *        Die Zehner- und die Hunderterstelle geben die Art des    *
        *        Fehlers, die Einerstelle den over- oder underflow an.    *
        *                                                                 *
        *        Error_Nr = 00 >>> kein Fehler                            *
        *        Error_Nr = 10 >>> durch 0 dividiert                      *
        *        Error_Nr = 20 >>> log einer negativen Zahl               *
        *        Error_Nr = 30 >>> asin, acos von abs(x) > 1              *
        *        Error_Nr = 40 >>> negative Basis a^b                     *
        *        Error_Nr = 50 >>> tan von Pi/2 * n                       *
        *        Error_Nr = 60 >>> Wurzel einer negativen Zahl            *
        *        Error_Nr = 70 >>> area - Argument out of range           *
        *        Error_Nr = 80 >>>                                        *
        *                                                                 *
        *******************************************************************)


   PROCEDURE Calc  (    root     :T_formel_pointer;
                        x        :REAL;
                    VAR y        :REAL;
                    VAR error_nr :INTEGER);

   (*.....................................................................*)

      FUNCTION Calculate (befehl  :INTEGER;
                          a, b    :REAL): REAL;

         CONST
            null = 1e-20;

         VAR
            res  :REAL;

      BEGIN
         res:= 0.0;
         CASE befehl OF
         C_plus: { a + b }
            (* Bei der Addition koennen folgende Fehler auftreten:
               - overflow error, ist aber sehr unwahrscheinlich ! *)
            res:= a + b;
         C_minus: { a - b }
            (* Bei der Subtraktion koennen folgende Fehler auftreten:
               - nach menschlichem ermessen keine Fehler moeglich *)
            res:= a - b;
         C_mal: { a * b }
            (* Bei der Multiplikation koennen folgende Fehler auftreten:
               - overflow error. (ist nicht abgesichert !) *)
            res:= a * b;
         C_geteilt: { a / b }
            (* Bei der Division koennen folgende Fehler auftreten:
               - Division durch 0;   (Grenze fuer 0 ist null)
               - overflow error  (nicht abgesichert !) *)
            IF ABS(b) > null THEN BEGIN
               res:= a / b;
            END
            ELSE BEGIN
               IF b = 0.0 THEN BEGIN
                  IF a >= 0 THEN BEGIN
                     error_nr:= 11;
                  END
                  ELSE BEGIN
                     error_nr:= 12;
                  END;
               END
               ELSE BEGIN
                  IF ((a>=0) AND (b>=0)) OR ((a<=0) AND (b<=0)) THEN BEGIN
                     error_nr:= 11;
                  END
                  ELSE BEGIN
                     error_nr:= 12;
                  END;
               END;
            END;
         C_hoch: { a ^ b }
            (* Bei der Exponentialfunktion koennen folgende Fehler auftreten:
               - overflow error (nicht abgesichert)
               - Basis a ist negativ  (ergibt kein Resultat)
               - b = 0 >>> Resultat ist 1; (Grenze fuer 0 ist null) *)
            IF (ABS(b) > null) THEN BEGIN
               IF (a > 0.0) THEN BEGIN
                  res:= EXP(LN(a)*b);
               END
               ELSE BEGIN
                  error_nr:= 40;
               END;
            END
            ELSE BEGIN
               res:= 1.0;
            END;
         C_sin: { sin (b) }
            (* Bei der Sinus-Funktion koennen folgende Fehler auftreten:
               - keine Fehler moeglich ! *)
            res:= SIN(b);
         C_cos: { cos (b) }
            (* Bei der Cosinus-Funktion koennen folgende Fehler auftreten:
               - keine Fehler moeglich ! *)
            res:= COS(b);
         C_tan: { tan (b) }
            (* Bei der Tangens-Funktion koennen folgende Fehler auftreten:
               - overflow error  (nicht abgesichert)
               - Tangens von Pi/2  (cos(b) = 0 kaum wahrscheinlich !) *)
            IF cos(b) <> 0.0 THEN BEGIN
               res:= SIN(b)/COS(b);
            END
            ELSE BEGIN
               error_nr:= 51;  (* Resultat = plus/minus unendlich *);
            END;
         C_sinh: { sinus hyperbolicus (b) }
            (* Bei der Sinh-Funktion koennen folgende Fehler auftreten:
               - keine Fehler moeglich ! *)
            res:= (EXP(b)-EXP(-b))/2;
         C_cosh: { cosinus hyperbolicus (b) }
            (* Bei der Cosh-Funktion koennen folgende Fehler auftreten:
               - keine Fehler moeglich ! *)
            res:= (EXP(b)+EXP(-b))/2;
         C_tanh: { tangens hyperbolicus (b) }
            (* Bei der Tanh-Funktion koennen folgende Fehler auftreten:
               - keine Fehler moeglich ! *)
            res:= (EXP(x)-EXP(-x))/(EXP(x)+EXP(-x));
         C_asin: { arcus sinus (b) }
            (* Bei der Asin-Funktion koennen folgende Fehler auftreten:
               - Argument dem Betrag nach > 1. Dies ergibt kein gueltiges Resultat *)
            IF ABS(b) < 1 THEN BEGIN
               res:= ARCTAN (b/(SQRT(1-SQR(b))));
            END
            ELSE BEGIN
               IF ABS(b) = 1 THEN BEGIN
                  IF b = 1 THEN BEGIN
                     res:= pi/2;
                  END
                  ELSE BEGIN
                     res:= -pi/2;
                  END;
               END
               ELSE BEGIN
                  error_nr:= 30;  (* asin(b) wenn abs(b) > 1 *)
               END;
            END;
         C_acos: { acos (b) }
            (* Bei der Acos-Funktion koennen folgende Fehler auftreten:
               - Argument dem Betrag nach > 1. Dies ergibt kein gueltiges Resultat *)
            IF abs(b) < 1 THEN BEGIN
               res:= ARCTAN (SQRT(1-SQR(b))/b);
               IF res <= 0 THEN res:= res+pi;
            END
            ELSE BEGIN
               IF ABS(b) = 1 THEN BEGIN
                  IF b = 1 THEN BEGIN
                     res:= 0;
                  END
                  ELSE BEGIN
                     res:= pi;
                  END;
               END
               ELSE BEGIN
                  error_nr:= 30;  (* acos(b) wenn abs(b) > 1 *)
               END;
            END;
         C_atan: { atan (b) }
            (* Bei der Atan-Funktion koennen folgende Fehler auftreten:
               - keine Fehler moeglich ! *)
            res:= ARCTAN (b);
         C_asinh: { area sinus hyperbolicus (b) }
            (* Bei der Asinh-Funktion koennen folgende Fehler auftreten:
               - keine Fehler moeglich ! *)
            res:= LN(b+SQRT(b*b+1));
         C_acosh: { area cosinus hyperbolicus (b) }
            (* Bei der Acosh-Funktion koennen folgende Fehler auftreten:
               - Argument b<1 >>> kein brauchbares Resultat ! *)
            IF b >= 1 THEN BEGIN
               res:= LN(b+SQRT(b*b-1));
            END
            ELSE BEGIN
               error_nr:= 70;
            END;
         C_atanh: { area tangens hyperbolicus (b) }
            (* Bei der Atanh-Funktion koennen folgende Fehler auftreten:
               - overflow error
               - Argument b dem Betrag nach groesser gleich 1 >>> no result *)
            IF ABS(b) < 1 THEN BEGIN
               res:= 0.5*LN((1+b)/(1-b));
            END
            ELSE BEGIN
               error_nr:= 70;
            END;
         C_log: { log (b) }
            (* Bei der Log-Funktion koennen folgende Fehler auftreten:
               - Argument b < 0 >>> no result
               - Argument b = 0 >>> Resultat ist minus unendlich *)
            IF b > 0 THEN BEGIN
               res:= LN(b)/LN(10);
            END
            ELSE BEGIN
               IF b = 0 THEN BEGIN
                  error_nr:= 22;
               END
               ELSE BEGIN
                  error_nr:= 20;
               END;
            END;
         C_ln: { ln (b) }
            (* Bei der Ln-Funktion koennen folgende Fehler auftreten:
               - Argument b < 0 >>> no result
               - Argument b = 0 >>> Resultat ist minus unendlich *)
            IF b > 0 THEN BEGIN
               res:= LN(b);
            END
            ELSE BEGIN
               IF b = 0 THEN BEGIN
                  error_nr:= 22;
               END
               ELSE BEGIN
                  error_nr:= 20;
               END;
            END;
         C_exp: { exp (b) }
            (* Bei der Exp-Funktion koennen folgende Fehler auftreten:
               - overflow error; (nicht abgesichert) *)
            res:= EXP(b);
         C_sqr:
            (* Bei der Square-Funktion koennen folgende Fehler auftreten:
               - overflow error; (nicht abgesichert) *)
            res:= SQR(b);
         C_sqrt:
            (* Bei der Squareroot-Funktion koennen folgende Fehler auftreten:
               - Argument b ist kleiner als 0 *)
            IF b >= 0 THEN BEGIN
               res:= SQRT(b);
            END
            ELSE BEGIN
               error_nr:= 60;
            END;
         C_abs:
            (* Bei der Abs-Funktion koennen folgende Fehler auftreten:
               - keine Fehler moeglich *)
            res:= ABS(b);
         C_frac:
            (* Bei der Frac-Funktion koennen folgende Fehler auftreten:
               - keine Fehler moeglich *)
            res:= FRAC(b);
         C_int:
            (* Bei der Int-Funktion koennen folgende Fehler auftreten:
               - keine Fehler moeglich *)
            res:= INT(b);
         END;
         Calculate:= res;
      END;

   (*.....................................................................*)

      FUNCTION Tree_Calc (branch :T_formel_pointer): REAL;

         VAR
            left_value, right_value  :REAL;

      BEGIN
         IF (branch = NIL) OR (error_nr <> 0) THEN BEGIN
            Tree_Calc:= 0.0;
         END
         ELSE BEGIN
            IF (branch^.left = nil) AND (branch^.right = nil) THEN BEGIN
               CASE branch^.befehl OF
                  C_var:    Tree_Calc:= branch^.zahl;
                  C_x:      Tree_Calc:= x;
                  C_pi:     Tree_Calc:= pi;
                  C_random: Tree_Calc:= RANDOM;
               END;
            END
            ELSE BEGIN
               left_value := Tree_Calc(branch^.left);
               right_value:= Tree_Calc(branch^.right);
               Tree_Calc:= Calculate (branch^.befehl, left_value, right_value);
            END;
         END;
      END;


   BEGIN
      error_nr:= 0;
      y:= Tree_Calc (root);
   END;


(*------------------------------------------------------------------------*)


  (************************************************************************
   *                                                                      *
   *  PROCEDURE  Translate_Formel                                         *
   *                                                                      *
   ************************************************************************
        *                                                                 *
        *  Formt die in einen String eingelesene Formel in einen          *
        *    Pointerbaum um.                                              *
        *  Eine unkorrekte Formel bewirkt die Rueckgabe der Fehler -      *
        *    position (error_pos) in dem Formelstring.                    *
        *  Die Formel wird in dieser Procedure bearbeitet und kommt im    *
        *    String selber veraendert zurueck. D.h. die Spaces werden     *
        *    herausgenommen und Grossbuchstaben werden in Kleinbuchstaben *
        *    umgewandelt.                                                 *
        *                                                                 *
        *******************************************************************)


   PROCEDURE Translate_Formel (VAR formula   :T_string80;
                               VAR root      :T_formel_pointer;
                               VAR error_pos :INTEGER);

      TYPE
         T_sortelement = RECORD
                            s_prioritaet :INTEGER;
                            s_befehl     :INTEGER;
                            s_zahl       :REAL;
                         END;
         T_sort_array = ARRAY [1..max_anz_befehle] OF T_sortelement;

      VAR
         top      :INTEGER;
         dim      :INTEGER;
         sort     :T_sort_array;

   (*---------------------------------------------------------------------*)

      FUNCTION Reformat (x :T_string80): T_string80;
         VAR
            index  :INTEGER;

         xx     :T_string80;
      BEGIN
         xx:= '';
         FOR index:= 1 TO length(x) DO BEGIN
            IF x[index] <> ' ' THEN BEGIN
               IF x[index] in ['A'..'Z'] THEN BEGIN
                 xx:= xx + chr(ord(x[index]) +32);
               END
               ELSE BEGIN
                 xx:= xx + x[index];
               END;
            END;
         END;
         Reformat:= xx;
      END;  (* of Reformat *)

   (*---------------------------------------------------------------------*)

      PROCEDURE Command_Test (    teststring :T_string80;
                              VAR x          :INTEGER;
                              VAR prioritaet :INTEGER;
                              VAR zahl       :REAL);
         VAR
            impossible :BOOLEAN;
            index      :INTEGER;

      BEGIN
         x:= 0;
         prioritaet:= 0;
         zahl:= 0.0;
         index:= 0;
         WHILE (index <= max_anz_befehle) AND (code[index] <> teststring) DO BEGIN
            index:= index +1;
         END;
         IF (index <= max_anz_befehle) THEN x:= index;
         IF x = 0 THEN BEGIN
            impossible:= false;
            Real_Test (teststring, zahl, impossible);
            IF NOT impossible THEN x:= 1;
         END;
         CASE x OF
            C_plus, C_minus:
               prioritaet:= 1;
            C_mal, C_geteilt:
               prioritaet:= 3;
            C_hoch:
               prioritaet:= 4;
            050..099:
               prioritaet:= 5;
            C_kauf, C_kzu:
               prioritaet:= 6;
            C_var, C_x, C_pi, C_random:
               prioritaet:= 7;
         END;
      END;  (* of Command_Test *)

   (*---------------------------------------------------------------------*)

      PROCEDURE Formel_in_Array (    formula :T_string80;
                                 VAR sort    :T_sort_array;
                                 VAR top     :INTEGER);

         TYPE
            T_modus = (zahl_m, wort_m, single_m, impossible_m);

         VAR
            formel_i      :INTEGER;
            formel_i_save :INTEGER;
            sort_i        :INTEGER;
            k_auf         :INTEGER;
            modus         :T_modus;


         PROCEDURE Get_Modus (VAR modus: T_modus);
         BEGIN
            modus:= impossible_m;
            CASE formula[formel_i] OF
            'x', '(', ')', '+', '-', '*', '/', '^' :
               BEGIN
                  modus:= single_m;
               END;
            '0'..'9', '.' :
               BEGIN
                  modus:= zahl_m;
               END;
            'a'.. 'z' :
               BEGIN
                  modus:= wort_m;
               END;
            END;
            IF modus = impossible_m THEN BEGIN
               error_pos:= formel_i;
            END;
         END;  (* of Get_Modus *)


         PROCEDURE Get_Next_Element (modus: T_modus);
            VAR
               next_one               :T_string80;
               exponent, dezimalpunkt :BOOLEAN;
         BEGIN
            next_one:= '';
            WITH sort[sort_i] DO BEGIN
               s_zahl:= 0.0;
               s_prioritaet:= 0;
               s_befehl:= 0;
               CASE modus OF
               single_m:
                  BEGIN
                     next_one:= formula[formel_i];
                     IF (next_one = '(') THEN BEGIN
                        k_auf:= k_auf +1;
                     END;
                     IF (next_one = ')') THEN BEGIN
                        IF k_auf <> 0 THEN BEGIN
                           k_auf:= k_auf -1;
                        END
                        ELSE BEGIN
                           error_pos:= formel_i;
                        END;
                     END;
                     formel_i:= formel_i +1;
                  END;
               zahl_m:
                  BEGIN
                     dezimalpunkt:= false;
                     exponent:= false;
                     REPEAT
                        CASE formula[formel_i] OF
                        '.':
                           BEGIN
                              IF (exponent) OR (dezimalpunkt) THEN BEGIN
                                 error_pos:= formel_i;
                              END
                              ELSE BEGIN
                                 dezimalpunkt:= true;
                              END;
                           END;
                        'e':
                           BEGIN
                              IF (exponent) THEN BEGIN
                                 error_pos:= formel_i;
                              END
                              ELSE BEGIN
                                 exponent:= true;
                                 IF (formel_i < dim)
                                    AND (formula[formel_i +1] IN ['+','-']) THEN BEGIN
                                    next_one:= next_one + formula[formel_i];
                                    formel_i:= formel_i +1;
                                 END;
                              END;
                           END;
                        '0'..'9':
                           BEGIN
                           END;
                        END;
                        IF error_pos = 0 THEN BEGIN
                           next_one:= next_one + formula[formel_i];
                           formel_i:= formel_i +1;
                        END;
                     UNTIL (error_pos <> 0) OR (formel_i > dim)
                            OR NOT (formula[formel_i] IN ['0'..'9', '.', 'e']);
                  END;
               wort_m:
                  BEGIN
                     REPEAT
                        next_one:= next_one + formula[formel_i];
                        formel_i:= formel_i +1;
                     UNTIL NOT (formula[formel_i] in ['a'..'z'])
                            OR (formel_i > dim);
                  END;
               impossible_m:
                  BEGIN
                     error_pos:= formel_i;
                  END;
               END;
               IF error_pos = 0 THEN BEGIN
                  (* noch kein Fehler, dann Kommando auswerten *)
                  Command_Test (next_one, s_befehl, s_prioritaet, s_zahl);
               END;
               IF (s_befehl = 0) AND (error_pos = 0) THEN BEGIN
                  (* kein Befehl in der gesuchten Art vorhanden ! *)
                  error_pos:= formel_i_save;
               END;
            END;
         END;  (* of Get_Next_Element *)

      (*..................................................................*)

         PROCEDURE Sequenz_Test;

            VAR
               last_one, aktuell  :INTEGER;


            PROCEDURE Null_Einfuegen;
            BEGIN
               sort[sort_i +1]:= sort[sort_i];
               sort[sort_i].s_zahl:= 0.0;
               sort[sort_i].s_prioritaet:= 7;
               sort[sort_i].s_befehl:= C_var;
               sort_i:= sort_i +1;
            END;


         BEGIN
            aktuell:= sort[sort_i].s_befehl;
            IF sort_i = 1 THEN BEGIN
               last_one:= 10;
            END
            ELSE BEGIN
               last_one:= sort[sort_i -1].s_befehl;
            END;
            CASE last_one OF
            C_error:      (* Kommt zum Zug nur wenn Fehler im Programm ! *)
               BEGIN
                  error_pos:= formel_i_save;
               END;
            C_var, C_x, C_pi, C_random, C_kzu:
               BEGIN
                  IF (aktuell IN [C_var, C_x, C_pi, C_random, C_kauf, 50..99]) THEN BEGIN
                     error_pos:= formel_i_save;
                  END;
               END;
            C_kauf:
               BEGIN
                  IF (aktuell IN [C_kzu, C_mal, C_geteilt, C_hoch]) THEN BEGIN
                     error_pos:= formel_i_save;
                  END
                  ELSE BEGIN
                     IF aktuell IN
                     [C_plus, C_minus]
                     THEN Null_Einfuegen;
                  END;
               END;
            C_plus, C_minus, C_mal, C_geteilt, C_hoch:
               BEGIN
                  IF (aktuell IN
                     [C_kzu, C_plus, C_minus, C_mal, C_geteilt, C_hoch]
                     ) THEN BEGIN
                     error_pos:= formel_i_save;
                  END;
               END;
            50..99:     (* Funktionen *)
               BEGIN
                  IF NOT (aktuell = C_kauf) THEN BEGIN
                     error_pos:= formel_i_save;
                  END;
               END;
            END; (* of CASE *)
         END;

      (*..................................................................*)

      BEGIN  (* Formel_in_Array *)
         top:= 0;
         (* Anfang Globale Variabeln *)
         formel_i:= 1;
         sort_i:= 0;
         k_auf:= 0;
         (* Ende Globale Variabeln *)
         REPEAT
            sort_i:= sort_i +1;
            formel_i_save:= formel_i;
            Get_Modus (modus);
            IF error_pos = 0 THEN Get_Next_Element (modus);
            IF error_pos = 0 THEN Sequenz_Test;
         UNTIL (error_pos <> 0) OR (formel_i > dim);
         IF (error_pos = 0) THEN BEGIN
            IF ((k_auf <> 0) OR
               (not(sort[sort_i].s_befehl in [C_var, C_x, C_pi, C_random, C_kzu])))
               AND (error_pos = 0) THEN BEGIN
               error_pos:= formel_i;
            END;
         END;
         IF error_pos = 0 THEN top:= sort_i;
      END;  (* of Formel_in_Array *)

   (*---------------------------------------------------------------------*)

      FUNCTION Formel_in_Tree (sort :T_sort_array; top: INTEGER): T_formel_pointer;
         VAR
            root   :T_formel_pointer;


         PROCEDURE Make_Tree (    low_i, high_i :INTEGER;
                              VAR hang_on       :T_formel_pointer);
            VAR
               high_priority_i  :INTEGER;
               part             :T_formel_pointer;


            FUNCTION High_Priority_Test (VAR low, high :INTEGER): INTEGER;
               VAR
                  n, k_auf, bef, pr     :INTEGER;
                  number, klammern      :INTEGER;
            BEGIN
               number:=low;
               klammern:=9999;
               k_auf:= 0;
               FOR n:= low TO high DO BEGIN
                  bef:= sort[n].s_befehl;
                  pr:= sort[n].s_prioritaet;
                  IF bef IN [C_kauf, C_kzu] THEN BEGIN
                     IF bef = C_kauf THEN BEGIN
                        k_auf:= k_auf +1;
                     END
                     ELSE BEGIN
                        k_auf:= k_auf -1;
                     END
                  END
                  ELSE BEGIN
                     IF ((k_auf <= klammern) AND
                        (pr <= sort[number].s_prioritaet)) OR
                        (k_auf < klammern) THEN BEGIN
                        number:=n;
                        klammern:=k_auf;
                     END;
                  END;
               END;
               For n:= 1 TO klammern DO BEGIN
                   low:=low+1;
                   high:=high-1;
               END;
               High_Priority_Test:= number;
            END;  (* of High_Priority_Test *)


         BEGIN
            new (part);
            hang_on:= part;
            part^.befehl:= 0;
            part^.zahl:= 0.0;
            part^.left:= nil;
            part^.right:= nil;
            high_priority_i:= High_Priority_Test (low_i, high_i);
            IF low_i < high_priority_i THEN BEGIN
               Make_Tree (low_i, high_priority_i -1, part^.left);
            END;
            IF high_i > high_priority_i THEN BEGIN
               Make_Tree (high_priority_i +1, high_i, part^.right);
            END;
            part^.befehl:= sort[high_priority_i].s_befehl;
            part^.zahl:= sort[high_priority_i].s_zahl;
         END; (* of Make_Tree *)


      BEGIN
         root:= nil;
         Make_Tree (1, top, root);
         Formel_in_Tree:= root;
      END; (* of Formel_in_Tree *)

   (*.....................................................................*)

      PROCEDURE Optimize_Formel (root  :T_formel_pointer);

         VAR
            y         :REAL;
            error_nr  :INTEGER;


         FUNCTION X_Includet (root :T_formel_pointer):BOOLEAN;
            VAR
               exists  :BOOLEAN;

            PROCEDURE Look (branch :T_formel_pointer);
               (* sucht nach Variabeln die nicht zum voraus eine Berechnung ermoeglichen *)
            BEGIN
               IF (NOT exists) AND (branch <> NIL) THEN BEGIN
                  IF (branch^.befehl IN [C_x, C_random]) THEN BEGIN
                      exists:= TRUE;
                  END
                  ELSE BEGIN
                     Look (branch^.left);
                     Look (branch^.right);
                  END;
               END;
            END;

         BEGIN
            exists:= FALSE;
            Look (root);
            X_Includet:= exists;
         END; (* of X_Includet *)


         PROCEDURE Optimize (root  :T_formel_pointer);
         BEGIN
            IF (root <> NIL) THEN BEGIN
               IF NOT (X_Includet (root)) THEN BEGIN
                  Calc (root, 0, y, error_nr);
                  IF error_nr = 0 THEN BEGIN
                  (* Hier sollten eigentlich noch Pointers geloescht werden *)
                     root^.left:= NIL;
                     root^.right:= NIL;
                     root^.befehl:= C_var;
                     root^.zahl:= y;
                  END;
               END
               ELSE BEGIN
                  Optimize (root^.left);
                  Optimize (root^.right);
               END;
            END;
         END;


      BEGIN
         y:= 0;
         error_nr:= 0;
         Optimize (root);
      END;

   (*.....................................................................*)

   BEGIN
      error_pos:= 0;
      root:= nil;
      formula:= Reformat (formula);
      dim:= length(formula);
      IF dim = 0 THEN BEGIN
        error_pos:= 1;
      END
      ELSE BEGIN
         Formel_in_Array (formula, sort, top);
         IF error_pos = 0 THEN BEGIN
            root:= Formel_in_Tree (sort, top);
            Optimize_Formel (root);
         END;
      END;
   END;  (* of Translate_Formel *)

(*------------------------------------------------------------------------*)


  (************************************************************************
   *                                                                      *
   *  PROCEDURE  Get_y_Margins                                            *
   *                                                                      *
   ************************************************************************
        *                                                                 *
        *  Berechnet den maximalen (bzw. minimalen Wert) der bei der      *
        *    spezifizierten (root) Formel im Bereich x_min bis x_max      *
        *    erreicht wird.                                               *
        *  In diesem Bereich werden <xmax> Punkte berechnet.              *
        *                                                                 *
        *                                                                 *
        *******************************************************************)


   PROCEDURE Get_y_Margins (    root         :T_formel_pointer;
                                x_min, x_max :REAL;
                            VAR y_min, y_max :REAL);

      VAR
         start       :BOOLEAN;
         index       :INTEGER;
         error_nr    :INTEGER;
         x, y, step  :REAL;

   BEGIN
      y_min:= 0;
      y_max:= 0;
      start:= false;
      x:= x_min;
      step:= (x_max - x_min)/(xmax -1)*3;
      FOR index:= 0 TO ROUND(xmax/3) DO BEGIN
         Calc (root, x, y, error_nr);
         IF error_nr = 0 THEN BEGIN
            IF NOT start THEN BEGIN
               start:= TRUE;
               y_min:= y;
               y_max:= y;
            END
            ELSE BEGIN
               IF (y < y_min) THEN y_min:= y;
               IF (y > y_max) THEN y_max:= y;
            END;
         END;
         x:= x + step;
      END;
   END;

(*------------------------------------------------------------------------*)


  (************************************************************************
   *                                                                      *
   *  PROCEDURE  Tab_Calc                                                 *
   *                                                                      *
   ************************************************************************
        *                                                                 *
        *  Diese Procedure berechnet eine Tabelle im Bereich x_min bis    *
        *    x_max mit der spezifizierten Funktion (root).                *
        *  Es werden <xmax> Punkte berechnet. (Im Array 000..<xmax>). Der *
        *    y-Array enthaelt nach der Berechnung Zahlen im Bereich von   *
        *    000...<ymax>.                                                *
        *  Ist das Formelresultat ausserhalb des y-Bereiches, wird bei    *
        *    Unterschreitung -1, bei Ueberschreitung <ymax> + 1           *
        *    abgespeichert.                                               *
        *    Ein unbrauchbares Resultat hat den Wert -2 zur Folge         *
        *                                                                 *
        *                                                                 *
        *******************************************************************)


   PROCEDURE Tab_Calc (    root         :T_formel_pointer;
                           x_min, x_max :REAL;
                           y_min, y_max :REAL;
                       VAR tabelle      :T_Tabelle);

      VAR
         error_nr, index :INTEGER;
         faktor, step    :REAL;
         line, x, y      :REAL;

   BEGIN
      x:= x_min;
      step:= (x_max-x_min)/(xmax-1);
      faktor:= (y_max-y_min)/(ymax);
      FOR index:= 0 TO xmax DO BEGIN
         Calc (root, x, y, error_nr);
         IF error_nr = 0 THEN BEGIN
            line:= (y-y_min)/faktor;
            IF line < 0 THEN line:= -1;
            IF line > ymax +1 THEN line:= ymax +1;
         END
         ELSE BEGIN
            IF (error_nr MOD 10) = 0 THEN line:= -2; (* kein brauchbares Resultat *)
            IF (error_nr MOD 10) = 1 THEN line:= -1; (* minus unendlich *)
            IF (error_nr MOD 10) = 2 THEN line:= ymax +1; (* plus unendlich *)
         END;
         tabelle[index]:= ROUND(line);
         x:= x + step;
      END;
   END;

