(******************************************************************************
*                                                                             *
*    package    HERCULES - G R A P H I K                                      *
*                                                                             *
*******************************************************************************
       *                                                                      *
       *  Version       : 1.4                                                 *
       *                                                                      *
       *  Date          : 21.11.87                                            *
       *                                                                      *
       *  Programmer    : M. Benz                                             *
       *                                                                      *
       *  Purpose:                                                            *
       *     Dieses Package beinhaltet alle Graphikprozeduren fr die Hercu-  *
       *     les-Grafikkarte.                                                 *
       *                                                                      *
       *  In/Out:                                                             *
       *     procedure Inittext                                               *
       *        Initialisiert den Graphikcontroller der Hercules Graphikkarte *
       *        fuer den Textmode.                                            *
       *                                                                      *
       *     procedure Initgraph                                              *
       *        Initialisiert den Graphikcontroller der Hercules Graphikkarte *
       *        fuer den Graphikbetrieb.                                      *
       *                                                                      *
       *     procedure Del_Graph_Screen                                       *
       *        Loescht den Graphikbildschirm.                                *
       *                                                                      *
       *     procedure Line (X1, Y1, X2, Y2: integer)                         *
       *        Zeichnet eine Linie auf dem Graphikbildschirm von (X1,Y1) zu  *
       *        (X2/Y2), wobei (X1/Y1) und (X2/Y2) die Anfangs- und Endpunkte *
       *        im Koordinatensystems sind.                                   *
       *                                                                      *
       *     procedure Del_Line (X1, Y1, X2, Y2: integer)                     *
       *        Wie Line, loescht aber die Linie.                             *
       *                                                                      *
       *     procedure Set_Point (X, Y: integer)                              *
       *        Setzt einen einzelnen Punkt auf dem Bildschirm.               *
       *                                                                      *
       *     procedure Del_Point (X, Y: integer)                              *
       *        Loescht einen einzelnen Punkt auf dem Bildschirm.             *
       *                                                                      *
       *     procedure Mark (X, Y: integer)                                   *
       *        Markiert ein Punkt auf dem Bildschirm mit einem Kreuz.        *
       *                                                                      *
       *     function Read_Point (X, Y : integer) : T_Status                  *
       *        Liest den Status (ein/aus) eines Punktes auf dem Bildschirm.  *
       *                                                                      *
       *     procedure Graph_Write (Stri: T_String80; Zeile, Spalte: integer) *
       *        Schreibt String auf den Graphikbildschirm.                    *
       *                                                                      *
       *     procedure Skala (Xvon, Xbis, Yvon, Ybis: real)                   *
       *        Zeichnet Achsenkreuz und am Rand des Schirmes die Skala.      *
       *                                                                      *
       ***********************************************************************)



   (***************************************************************************
   *                                                                          *
   *   procedure Inittext                                                     *
   *                                                                          *
   ****************************************************************************
         *                                                                    *
         *  Date        : 16.03.87                                            *
         *  Programmer  : M. Benz                                             *
         *                                                                    *
         *  Purpose:                                                          *
         *     Diese Prozedur initialisiert den Viedocontroller MC 6845       *
         *     fuer den Textmode.                                             *
         *                                                                    *
         *  In/Out:                                                           *
         *     procedure Inittext                                             *
         *                                                                    *
         *********************************************************************)

   procedure Inittext;
   begin
       Port[$03B8] := $20;    (* Textfreigabe an Display-Modecontrollport    *)
                              (* und Dunkeltastung des Schirmes              *)
       Port[$03B4] := 0;      (* Initialisierung des Videocontrollers        *)
       Port[$03B5] := 97;     (* MC 6845 CRTC                                *)
       Port[$03B4] := 1;
       Port[$03B5] := 80;
       Port[$03B4] := 2;
       Port[$03B5] := 82;
       Port[$03B4] := 3;
       Port[$03B5] := 15;
       Port[$03B4] := 4;
       Port[$03B5] := 25;
       Port[$03B4] := 5;
       Port[$03B5] := 6;
       Port[$03B4] := 6;
       Port[$03B5] := 25;
       Port[$03B4] := 7;
       Port[$03B5] := 25;
       Port[$03B4] := 8;
       Port[$03B5] := 2;
       Port[$03B4] := 9;
       Port[$03B5] := 13;
       Port[$03B4] := 10;
       Port[$03B5] := 11;
       Port[$03B4] := 11;
       Port[$03B5] := 12;
       Delay (300);           (* Delay bis Bild stabil    *)
       Port[$03B8] := $28;    (* Helltastung des Schirmes *)
   end;   (* Inittext *)



   (***************************************************************************
   *                                                                          *
   *   procedure Initgraph                                                    *
   *                                                                          *
   ****************************************************************************
         *                                                                    *
         *  Date        : 16.03.87                                            *
         *  Programmer  : M. Benz                                             *
         *                                                                    *
         *  Purpose:                                                          *
         *     Diese Prozedur initialisiert den Videocontroller MC 6845       *
         *     fuer den Graphikbetrieb.                                       *
         *                                                                    *
         *  In/Out:                                                           *
         *     procedure Initgraph                                            *
         *                                                                    *
         *********************************************************************)

   procedure Initgraph;
   begin
       Port[$03B8] := $82;    (* Graphikfreigabe an Display-Modecontrollport *)
                              (* und Dunkeltastung des Schirmes              *)
       Port[$03B4] := 0;      (* Initialisierung des Videocontrollers        *)
       Port[$03B5] := 53;     (* MC 6845 CRTC                                *)
       Port[$03B4] := 1;
       Port[$03B5] := 45;
       Port[$03B4] := 2;
       Port[$03B5] := 46;
       Port[$03B4] := 3;
       Port[$03B5] := 8;
       Port[$03B4] := 4;
       Port[$03B5] := 91;
       Port[$03B4] := 5;
       Port[$03B5] := 2;
       Port[$03B4] := 6;
       Port[$03B5] := 87;
       Port[$03B4] := 7;
       Port[$03B5] := 87;
       Port[$03B4] := 8;
       Port[$03B5] := 2;
       Port[$03B4] := 9;
       Port[$03B5] := 3;
       Port[$03B4] := 10;
       Port[$03B5] := 0;
       Port[$03B4] := 11;
       Port[$03B5] := 0;
       Delay (150);           (* Delay bis Bild stabil    *)
       Port[$03B8] := $8A;    (* Helltastung des Schirmes *)
   end;   (* Initgraph *)



   (***************************************************************************
   *                                                                          *
   *   procedure Del_Graph_Screen                                             *
   *                                                                          *
   ****************************************************************************
         *                                                                    *
         *  Date        : 14.03.87                                            *
         *  Programmer  : M. Benz                                             *
         *                                                                    *
         *  Purpose:                                                          *
         *     Loescht den Graphischen Bildschirm.                            *
         *                                                                    *
         *  In/Out:                                                           *
         *     procedure Del_Graph_Screen                                     *
         *                                                                    *
         *********************************************************************)

   procedure Del_Graph_Screen;
       var
           I      : integer;
   begin
       for I:=0 to $7FFF do begin
           Memw[$B800:I] := 0;
       end;
   end;   (* Del_Graph_Screen *)



   (***************************************************************************
   *                                                                          *
   *   procedure Set_Point                                                    *
   *                                                                          *
   ****************************************************************************
         *                                                                    *
         *  Date        : 16.03.87                                            *
         *  Programmer  : M. Benz                                             *
         *                                                                    *
         *  Purpose:                                                          *
         *     Setzt im Graphikmode einen Punkt auf dem Bildschirm.           *
         *                                                                    *
         *  In/Out:                                                           *
         *     procedure Set_Point (X, Y: integer)                            *
         *        X, Y : Koordinaten des Punktes auf dem Bildschirm.          *
         *                                                                    *
         *********************************************************************)

   procedure Set_Point (X, Y: integer);
       var
           Adresse   : integer;
           By        : byte;
   begin
       if (X>=Xmin) and (X<=Xmax) and (Y>=Ymin) and (Y<=Ymax) then begin
           Y:= Ymax-Y;
           X:= X-Xmin;
           Adresse:= (Y mod 4)*$2000+trunc((90*int(Y/4))+int(X/8));
           By:= Mem[$B800:Adresse];
           case (X mod 8) of
               0: By:= By or $80;
               1: By:= By or $40;
               2: By:= By or $20;
               3: By:= By or $10;
               4: By:= By or $08;
               5: By:= By or $04;
               6: By:= By or $02;
               7: By:= By or $01;
           end;
           Mem[$B800:Adresse]:= By;
       end;
   end;  (* Set_Point *)



   (***************************************************************************
   *                                                                          *
   *   procedure Line                                                         *
   *                                                                          *
   ****************************************************************************
         *                                                                    *
         *  Date        : 11.03.87                                            *
         *  Programmer  : M. Benz                                             *
         *                                                                    *
         *  Purpose:                                                          *
         *     Zeichnet eine Linie auf den Bildschirm.                        *
         *                                                                    *
         *  In/Out:                                                           *
         *     procedure Line (X1, Y1, X2, Y2: integer)                       *
         *        X1, Y1 : Koordinaten des Anfangspunktes                     *
         *        X2, Y2 : Koordinaten des Endpunktes                         *
         *                                                                    *
         *********************************************************************)

   procedure Line (X1, Y1, X2, Y2: integer);
       var
           X, Y, SignX, SignY, I     : integer;
   begin
       if X1<X2 then SignX:=1
       else SignX:=-1;
       if Y1<Y2 then SignY:=1
       else SignY:=-1;
       if X1=X2 then begin
           for X:=0 to abs(Y1-Y2) do begin
               Y:=Y1+SignY*X;
               Set_Point (X1, Y);
           end;
       end
       else begin
           if abs(X1-X2) > abs(Y1-Y2) then begin
               for X:=0 to abs(X1-X2) do begin
                   Y:=round(Y1+abs(Y1-Y2)/abs(X1-X2)*SignY*X);
                   Set_Point (X1+X*SignX, Y);
               end;
           end
           else begin
               for Y:=0 to abs (Y1-Y2) do begin
                   X:=round(X1+abs(X1-X2)/abs(Y1-Y2)*SignX*Y);
                   Set_Point (X, Y1+Y*SignY);
               end;
           end;
       end;
   end;   (* Line *)



   (***************************************************************************
   *                                                                          *
   *   procedure Del_Point                                                    *
   *                                                                          *
   ****************************************************************************
         *                                                                    *
         *  Date        : 11.03.87                                            *
         *  Programmer  : M. Benz                                             *
         *                                                                    *
         *  Purpose:                                                          *
         *     Diese Prozedur loescht im Graphikmode einen Punkt auf dem      *
         *     Bildschirm.                                                    *
         *                                                                    *
         *  In/Out:                                                           *
         *     procedure Del_Point (X, Y: integer)                            *
         *        X, Y: Koordinaten des Punktes auf dem Bildschirm            *
         *                                                                    *
         *********************************************************************)

   procedure Del_Point (X, Y: integer);
       var
           Adresse     : integer;
           By          : byte;
   begin
       if (X>=Xmin) and (X<=Xmax) and (Y>=Ymin) and (Y<=Ymax) then begin
           Y:=Ymax-Y;
           X:=X-Xmin;
           Adresse:=(Y mod 4)*$2000+trunc((90*int(Y/4))+int(X/8));
           By:=Mem[$B800:Adresse];
           case (X mod 8) of
               0: By:=By and $7F;
               1: By:=By and $BF;
               2: By:=By and $DF;
               3: By:=By and $EF;
               4: By:=By and $F7;
               5: By:=By and $FB;
               6: By:=By and $FD;
               7: By:=By and $FE;
           end;
           Mem[$B800:Adresse]:=By;
       end;
   end;  (* Del_Point *)



   (***************************************************************************
   *                                                                          *
   *   procedure Del_Line                                                     *
   *                                                                          *
   ****************************************************************************
         *                                                                    *
         *  Date        : 11.03.87                                            *
         *  Programmer  : M. Benz                                             *
         *                                                                    *
         *  Purpose:                                                          *
         *     Loescht eine Linie auf dem Bildschirm.                         *
         *                                                                    *
         *  In/Out:                                                           *
         *     procedure Del_Line (X1, Y1, X2, Y2: integer)                   *
         *        X1, Y1: Koordinaten des Anfangspunktes                      *
         *        X2, Y2: Koordinaten des Endpunktes                          *
         *                                                                    *
         *********************************************************************)

   procedure Del_Line (X1, Y1, X2, Y2: integer);
       var
           X, Y, SignX, SignY   : integer;
   begin
       if X1<X2 then SignX:=1
       else SignX:=-1;
       if Y1<Y2 then SignY:=1
       else SignY:=-1;
       if X1 = X2 then begin
           for X:=0 to abs (Y1-Y2) do begin
               Y:=Y1+SignY*X;
               Del_Point (X1, Y);
           end;
       end
       else begin
           if abs(X1-X2) > abs(Y1-Y2) then begin
               for X:=0 to abs (X1-X2) do begin
                   Y:=round (Y1+abs(Y1-Y2)/abs(X1-X2)*SignY*X);
                   Del_Point (X1+X*SignX, Y);
               end;
           end
           else begin
               for Y:=0 to abs(Y1-Y2) do begin
                   X:=round (X1+abs(X1-X2)/abs(Y1-Y2)*SignX*Y);
                   Del_Point (X, Y1+Y*SignY);
               end;
           end;
       end;
   end;  (* Del_Line*)



   (***************************************************************************
   *                                                                          *
   *   procedure Mark_Point                                                   *
   *                                                                          *
   ****************************************************************************
         *                                                                    *
         *  Date        : 11.03.87                                            *
         *  Programmer  : M. Benz                                             *
         *                                                                    *
         *  Purpose:                                                          *
         *     Markiert einen Punkt auf dem Bildschirm.                       *
         *                                                                    *
         *  In/Out:                                                           *
         *     procedure Mark_Point (X, Y: integer)                           *
         *        X, Y : Koordinaten des zu markierenden Punktes.             *
         *                                                                    *
         *********************************************************************)

   procedure Mark_Point (X, Y: integer);
       var
           I               : integer;
   begin
       for I:=-2 to 2 do begin
           Set_Point (X+I, Y+I);
           Set_Point (X-I, Y+I);
       end;
   end;   (* Mark_Point *)



   (***************************************************************************
   *                                                                          *
   *   function Read_Point                                                    *
   *                                                                          *
   ****************************************************************************
         *                                                                    *
         *  Date        : 11.03.87                                            *
         *  Programmer  : M. Benz                                             *
         *                                                                    *
         *  Purpose:                                                          *
         *     Gibt den Zustand (On/Off) eines Bildschirmpunktes zurueck.     *
         *                                                                    *
         *  In/Out:                                                           *
         *     function Read_Point (X, Y: integer): T_Status                  *
         *        X, Y: Bildschirmkoordinaten des Punktes.                    *
         *                                                                    *
         *********************************************************************)

   function Read_Point (X, Y: integer): T_Status;
       var
           Address       : integer;
           By            : byte;
   begin
       if (X>=Xmin) and (X<=Xmax) and (Y>=Ymin) and (Y<=Ymax) then begin
           Y:=Ymax-Y;
           X:=X-Xmin;
           Address:=(Y mod 4)*$2000+trunc((90*int(Y/4))+int(X/8));
           By:=mem[$B800:Address];
           case (X mod 8) of
               0: By:=By and $80;
               1: By:=By and $40;
               2: By:=By and $20;
               3: By:=By and $10;
               4: By:=By and $08;
               5: By:=By and $04;
               6: By:=By and $02;
               7: By:=By and $01;
           end;
           if By=0 then Read_Point:=Off
           else Read_Point:=On;
       end;
   end;   (* Read_Point *)



   (***************************************************************************
   *                                                                          *
   *   procedure Graph_Write                                                  *
   *                                                                          *
   ****************************************************************************
         *                                                                    *
         *  Date        : 12.03.87                                            *
         *  Programmer  : M. Benz                                             *
         *                                                                    *
         *  Purpose:                                                          *
         *     Diese Prozedur schreibt einen String auf den Graphikbildschirm *
         *                                                                    *
         *  In/Out:                                                           *
         *     procedure Graph_Write (Stri: T_String80; Zeile,                *
         *                            Spalte: integer)                        *
         *         Stri          : zu schreibender Text                       *
         *         Zeile, Spalte : Zeile und Spalte des ersten Zeichens       *
         *                                                                    *
         *********************************************************************)

   procedure Graph_Write (Stri: T_String80; Zeile, Spalte: integer);
       var
           I, Leng             : integer;
           Ch                  : char;
   begin
      Leng:=length (Stri);
      if (Zeile>=0) and (Zeile<=Zmax) and
         (Spalte>=0) and (Spalte+Leng<=Smax) then begin
         for I:=0 to Leng do begin
            Ch:=Stri[I];
            case Ch of
               '0' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$38;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$4C;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$54;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$64;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$38;
                     end;
               '1' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$30;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$38;
                     end;
               '2' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$38;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$04;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$08;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$20;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$7C;
                     end;
               '3' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$7C;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$08;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$38;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$04;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$04;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$78;
                     end;
               '4' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$08;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$18;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$28;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$48;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$7C;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$08;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$08;
                     end;
               '5' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$7C;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$78;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$04;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$04;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$38;
                     end;
               '6' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$18;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$20;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$78;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$38;
                     end;
               '7' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$7C;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$04;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$08;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$10;
                     end;
               '8' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$38;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$38;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$38;
                     end;
               '9' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$38;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$3C;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$04;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$08;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$30;
                     end;
               'A' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$28;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$7C;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$44;
                     end;
               'B' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$78;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$24;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$24;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$38;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$24;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$24;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$78;
                     end;
               'C' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$38;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$38;
                     end;
               'D' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$78;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$24;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$24;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$24;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$24;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$24;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$78;
                     end;
               'E' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$7C;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$78;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$7C;
                     end;
               'F' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$7C;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$78;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$40;
                     end;
               'G' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$3C;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$4C;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$38;
                     end;
               'H' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$7C;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$44;
                     end;
               'I' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$38;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$38;
                     end;
               'J' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$1C;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$08;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$08;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$08;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$08;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$48;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$30;
                     end;
               'K' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$48;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$50;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$60;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$50;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$48;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$44;
                     end;
               'L' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$7C;
                     end;
               'M' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$6C;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$54;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$54;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$44;
                     end;
               'N' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$64;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$54;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$4C;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$44;
                     end;
               'O' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$38;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$38;
                     end;
               'P' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$78;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$78;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$40;
                     end;
               'Q' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$38;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$54;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$48;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$34;
                     end;
               'R' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$78;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$78;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$50;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$48;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$44;
                     end;
               'S' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$38;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$38;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$04;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$38;
                     end;
               'T' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$7C;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$10;
                     end;
               'U' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$38;
                     end;
               'V' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$28;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$28;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$10;
                     end;
               'W' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$54;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$54;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$6C;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$44;
                     end;
               'X' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$28;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$28;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$44;
                     end;
               'Y' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$28;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$10;
                     end;
               'Z' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$7C;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$04;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$08;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$20;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$7C;
                     end;
               '(' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$08;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$20;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$20;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$20;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$08;
                     end;
               ')' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$20;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$08;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$08;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$08;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$20;
                     end;
               '+' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$7C;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$00;
                     end;
               '-' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$7C;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$00;
                     end;
               '*' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$54;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$38;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$54;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$00;
                     end;
               '/' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$04;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$08;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$20;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$40;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$00;
                     end;
               '.' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$18;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$18;
                     end;
               '^' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$10;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$28;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$44;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$00;
                     end;
               ' ' : begin
                         Mem[$B800:      $B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$2000+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$4000+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$6000+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$005A+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$205A+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$405A+$B4*Zeile+Spalte+I]:=$00;
                         Mem[$B800:$605A+$B4*Zeile+Spalte+I]:=$00;
                     end;
               else;
            end; (* of case *)
         end; (* of for *)
      end; (* of if *)
   end;   (* Graph_Write *)



   (***************************************************************************
   *                                                                          *
   *   procedure Skala                                                        *
   *                                                                          *
   ****************************************************************************
         *                                                                    *
         *  Date        : 21.11.87                                            *
         *  Programmer  : M. Benz                                             *
         *                                                                    *
         *  Purpose:                                                          *
         *     Zeichnet Achsenkreuz und am Rand des Schirmes die Skala        *
         *                                                                    *
         *  In/Out:                                                           *
         *     procedure Skala (Xvon, Xbis, Yvon, Ybis: real)                 *
         *                                                                    *
         *********************************************************************)

   procedure Skala (Xvon, Xbis, Yvon, Ybis: real);
       var
           Xrange, Xresol, Xstep,
           Yrange, Yresol, Ystep, I                 : real;
           X0, Y0, Xexp, Yexp, Pos                  : integer;
           Stri, ExpS                               : T_String80;
           ExpB                                     : boolean;
   begin
       Xrange:=Xbis-Xvon; Yrange:=Ybis-Yvon;
       if Xbis>0 then Xexp:=trunc(ln(Xbis)/Ln10)     (* log (Xbis)  *)
       else Xexp:=trunc(ln(-Xvon)/Ln10);
       if Xexp<0 then Xexp:=Xexp-1;
       if Ybis>0 then Yexp:=trunc(ln(Ybis)/Ln10)     (* log (Ybis) *)
       else Yexp:=trunc(ln(-Yvon)/Ln10);
       if Yexp<0 then Yexp:=Yexp-1;
       Xresol:=exp((Xexp-1)*Ln10);                   (* 10^(Xexp-1) *)
       Yresol:=exp((Yexp-1)*Ln10);                   (* 10^(Yexp-1) *)

       (* Schrittweite der Teilstriche berechnen *)
       Xstep:=Xrange/20; Ystep:=Yrange/20;
       if Xstep<=Xresol then Xstep:=Xresol
       else if Xstep<=2*Xresol then Xstep:=2*Xresol
       else if Xstep<=5*Xresol then Xstep:=5*Xresol
       else Xstep:=10*Xresol;
       if Ystep<Yresol then Ystep:=Yresol
       else if Ystep<=2*Yresol then Ystep:=2*Yresol
       else if Ystep<=5*Yresol then Ystep:=5*Yresol
       else Ystep:=10*Yresol;

       if Xrange>=Xresol then X0:=round(-Xvon/Xrange*(Xmax+1))
       else X0:=-1;
       if Yrange>=Yresol then Y0:=round(-Yvon/Yrange*(Ymax+1))
       else Y0:=-1;
       if Y0>0 then Line (0, Y0, Xmax, Y0);     (* Achsenkreuz zeichnen *)
       if X0>0 then Line (X0, 0, X0, Ymax);
       Line (0, 0, Xmax, 0);                    (* Skalenrand zeichnen  *)
       Line (0, 0, 0, Ymax);

       (* Teilstriche auf der X-Achse eintragen *)
       I:=0;
       Pos:=X0+round(((Xmax+1)/Xrange)*I);
       while Pos<=Xmax do begin
           if Pos>=0 then Line (Pos, -4, Pos, 0);
           I:=I+Xstep;
           Pos:=X0+round(((Xmax+1)/Xrange)*I);
       end;
       I:=-Xstep;
       Pos:=X0+round(((Xmax+1)/Xrange)*I);
       while Pos>=0 do begin
           Line (Pos, -4, Pos, 0);
           I:=I-Xstep;
           Pos:=X0+round(((Xmax+1)/Xrange)*I);
       end;

       (* Teilstriche auf der Y-Achse eintragen *)
       I:=0;
       Pos:=Y0+round(((Ymax+1)/Yrange)*I);
       while Pos<=Ymax do begin
           if Pos>=0 then Line (-4, Pos, 0, Pos);
           I:=I+Ystep;
           Pos:=Y0+round(((Ymax+1)/Yrange)*I);
       end;
       I:=-Ystep;
       Pos:=Y0+round(((Ymax+1)/Yrange)*I);
       while Pos>=0 do begin
           Line (-4, Pos, 0, Pos);
           I:=I-Ystep;
           Pos:=Y0+round(((Ymax+1)/Yrange)*I);
       end;

       (* Exponentenschreibweise auf X-Skala *)
       ExpB:=false;
       if (Xexp>2) or (Xexp<-1) then begin
           ExpB:=true;
           str (Xexp:1, ExpS);
           Xexp:=0;
           Xrange:=Xrange/10/Xresol;
           Xstep:=Xstep/10/Xresol;
       end;

       (* 0.0 und Vorzeichen auf X-Skala eintragen *)
       if (Xrange/Xstep<=12) then Xstep:=Xstep/2;
       if (Xrange/Xstep<=12) then Xstep:=Xstep/2;
       if Xexp=2 then Xexp:=1;
       if X0>=0 then begin
           str (0.0:4:1-Xexp, Stri);
           Graph_Write (copy(Stri,2,3), Zmax, round((X0-Xmin)/8)-2);
       end;
       if (X0>16) and (X0<Xmax-16) then begin
           Pos:=round((X0-Xmin)/8+((Xmax+1)/Xrange)*2*Xstep/8)-2;
           Graph_Write ('+', Zmax, Pos);
           Pos:=round((X0-Xmin)/8-((Xmax+1)/Xrange)*2*Xstep/8)-2;
           Graph_Write ('-', Zmax, Pos);
       end;

       (* X-Skala beschriften *)
       I:=4*Xstep;
       Pos:=round((X0-Xmin)/8+((Xmax+1)/Xrange)*I/8)-2;
       while Pos<Smax do begin
           str (abs(I):4:1-Xexp, Stri);
           if Pos>2 then Graph_Write (copy(Stri,2,3), Zmax, Pos);
           I:=I+4*Xstep;
           Pos:=round((X0-Xmin)/8+((Xmax+1)/Xrange)*I/8)-2;
       end;
       if ExpB and (X0<Xmax-16) then begin     (* Exponent schreiben *)
           I:=I-6*Xstep;
           Pos:=round((X0-Xmin)/8+((Xmax+1)/Xrange)*I/8)-2;
           Graph_Write ('E'+ExpS, Zmax, Pos);
       end;
       I:=-4*Xstep;
       Pos:=round((X0-Xmin)/8+((Xmax+1)/Xrange)*I/8)-2;
       while Pos>2 do begin
           str (abs(I):4:1-Xexp, Stri);
           Graph_Write (copy(Stri,2,3), Zmax, Pos);
           I:=I-4*Xstep;
           Pos:=round((X0-Xmin)/8+((Xmax+1)/Xrange)*I/8)-2;
       end;
       if ExpB and (X0>=Xmax-16) then begin    (* Exponent schreiben *)
           I:=I+6*Xstep;
           Pos:=round((X0-Xmin)/8+((Xmax+1)/Xrange)*I/8)-2;
           Graph_Write ('-E'+ExpS, Zmax, Pos);
       end
       else if X0>=Xmax-16 then begin
           I:=I+6*Xstep;
           Pos:=round((X0-Xmin)/8+((Xmax+1)/Xrange)*I/8);
           Graph_Write ('-',Zmax, Pos);
       end;

       (* Exponentenschreibweise auf Y-Skala *)
       ExpB:=false;
       if (Yexp>2) or (Yexp<-1) then begin
           ExpB:=true;
           str (Yexp:1, ExpS);
           Yexp:=0;
           Yrange:=Yrange/10/Yresol;
           Ystep:=Ystep/10/Yresol;
       end;

       (* 0.0 und Vorzeichen auf Y-Skala eintragen *)
       if (Yrange/Ystep<=12) then Ystep:=Ystep/2;
       if (Yrange/Ystep<=12) then Ystep:=Ystep/2;
       if Yexp=2 then Yexp:=1;
       if Y0>=0 then begin
           str (0.0:4:1-Yexp, Stri);
           Graph_Write (copy(Stri,2,3), Zmax-round((Y0-Ymin)/8)+1, 0);
       end;
       if (Y0>16) and (Y0<Ymax-16) then begin
           Pos:=Zmax-round((Y0-Ymin)/8+((Ymax+1)/Yrange)*2*Ystep/8)+1;
           Graph_Write ('+', Pos, 1);
           Pos:=Zmax-round((Y0-Ymin)/8-((Ymax+1)/Yrange)*2*Ystep/8)+1;
           Graph_Write ('-', Pos, 1);
       end;

       (* Y-Skala beschriften *)
       I:=4*Ystep;
       Pos:=Zmax-round((Y0-Ymin)/8+((Ymax+1)/Yrange)*I/8)+1;
       while Pos>=0 do begin
           str (abs(I):4:1-Yexp, Stri);
           if Pos<=Zmax-1 then Graph_Write (copy(Stri,2,3), Pos, 0);
           I:=I+4*Ystep;
           Pos:=Zmax-round((Y0-Ymin)/8+((Ymax+1)/Yrange)*I/8)+1;
       end;
       if ExpB and (Y0<Ymax-16) then begin        (* Exponent schreiben *)
           I:=I-5*Ystep;
           Pos:=Zmax-round((Y0-Ymin)/8+((Ymax+1)/Yrange)*I/8)+1;
           Graph_Write ('E'+ExpS, Pos, 0);
       end;
       I:=-4*Ystep;
       Pos:=Zmax-round((Y0-Ymin)/8+((Ymax+1)/Yrange)*I/8)+1;
       while Pos<=Zmax-1 do begin
           str (abs(I):4:1-Yexp, Stri);
           Graph_Write (copy(Stri,2,3), Pos, 0);
           I:=I-4*Ystep;
           Pos:=Zmax-round((Y0-Ymin)/8+((Ymax+1)/Yrange)*I/8)+1;
       end;
       if ExpB and (Y0>=Ymax-16) then begin       (* Exponent schreiben *)
           I:=I+5*Ystep;
           Pos:=Zmax-round((Y0-Ymin)/8+((Ymax+1)/Yrange)*I/8)+1;
           Graph_Write ('E'+ExpS, Pos, 0);
           Graph_Write ('-', Pos-1, 1);
       end
       else if Y0>=Ymax-16 then begin
           I:=I+5*Ystep;
           Pos:=Zmax-round((Y0-Ymin)/8+((Ymax+1)/Yrange)*I/8)+1;
           Graph_Write ('-',Pos,1);
       end;

   end;  (* Skala *)

(******************************************************************************
*                                                                             *
*    End of package Graphik                                                   *
*                                                                             *
******************************************************************************)
