(**********************************************************************
*                                                                     *
*   module   I B M - G R A P H I C                                    *
*                                                                     *
***********************************************************************
    *                                                                 *
    *   Version       : 2.00                                          *
    *                                                                 *
    *   Date          : 88-03-03                                      *
    *                                                                 *
    *   Programmer    : M. Benz                                       *
    *                                                                 *
    *   Purpose:                                                      *
    *       Dieses Modul beinhaltet alle Graphikprozeduren fr die    *
    *       IBM-Graphikkarte.                                         *
    *                                                                 *
    *   In/Out:                                                       *
    *       procedure Inittext                                        *
    *           Initialisiert die Graphikkarte fr den Textmode.      *
    *                                                                 *
    *       procedure Initgraph                                       *
    *           Initialisiert die Graphikkarte fr den Graphikbetrieb.*
    *                                                                 *
    *       procedure Line (X1, Y1, X2, Y2: integer)                  *
    *           Zeichnet eine Linie auf dem Graphikbildschirm von     *
    *           (X1,Y1) zu (X2/Y2), wobei (X1/Y1) und (X2/Y2) die     *
    *           Anfangs- und Endpunkte im Koordinatensystems sind.    *
    *                                                                 *
    *       procedure Set_Point (X, Y: integer)                       *
    *           Setzt einen einzelnen Punkt auf dem Bildschirm.       *
    *                                                                 *
    *       procedure Del_Point (X, Y: integer)                       *
    *           Lscht einen einzelnen Punkt auf dem Bildschirm.      *
    *                                                                 *
    *       procedure Mark (X, Y: integer)                            *
    *           Markiert ein Punkt auf dem Bildschirm mit einem Kreuz.*
    *                                                                 *
    *       function Read_Point (X, Y : integer) : T_Status           *
    *           Liest den Status (ein/aus) eines Punktes auf dem      *
    *           Bildschirm.                                           *
    *                                                                 *
    ******************************************************************)

module Graphic;

    const
        Xmin    =  40;
        Xmax    = 600;
        Ymin    =  20;
        Ymax    = 200;
    type
        T_Status=(On, Off);

    procedure Int10 (var Ax, Bx, Cx, Dx: word);             extern;


    (******************************************************************
    *                                                                 *
    *   procedure Inittext                                            *
    *                                                                 *
    *******************************************************************
        *                                                             *
        *   Date        : 88-03-03                                    *
        *   Programmer  : M. Benz                                     *
        *                                                             *
        *   Purpose:                                                  *
        *       Diese Prozedur initialisiert die Graphikkarte fr den *
        *       Textmode.                                             *
        *                                                             *
        *   In/Out:                                                   *
        *       procedure Inittext                                    *
        *                                                             *
        **************************************************************)

    procedure Inittext;
        var
            Ax, Bx, Cx, Dx      : word;
    begin
        Ax:=#0002;
        Int10 (Ax, Bx, Cx, Dx);
    end {Inittext};



    (******************************************************************
    *                                                                 *
    *   procedure Initgraph                                           *
    *                                                                 *
    *******************************************************************
        *                                                             *
        *   Date        : 88-03-03                                    *
        *   Programmer  : M. Benz                                     *
        *                                                             *
        *   Purpose:                                                  *
        *       Diese Prozedur initialisiert die Graphikkarte fr den *
        *       Graphikbetrieb.                                       *
        *                                                             *
        *   In/Out:                                                   *
        *       procedure Initgraph                                   *
        *                                                             *
        **************************************************************)

    procedure Initgraph;
        var
            Ax, Bx, Cx, Dx      : word;
    begin
        Ax:=#0006;
        Int10 (Ax, Bx, Cx, Dx);
    end {Initgraph};



    (******************************************************************
    *                                                                 *
    *   procedure Set_Point                                           *
    *                                                                 *
    *******************************************************************
        *                                                             *
        *   Date        : 88-03-03                                    *
        *   Programmer  : M. Benz                                     *
        *                                                             *
        *   Purpose:                                                  *
        *       Setzt im Graphikmode einen Punkt auf dem Bildschirm.  *
        *                                                             *
        *   In/Out:                                                   *
        *       procedure Set_Point (X, Y: integer)                   *
        *           X, Y : Koordinaten des Punktes auf dem Bildschirm.*
        *                                                             *
        **************************************************************)

    procedure Set_Point (X, Y: integer);
        var
            Ax, Bx, Cx, Dx      : word;
    begin
        if (X>=Xmin) and (X<=Xmax) and (Y>=Ymin) and 
           (Y<=Ymax) then begin
            Dx:=Ymax-Y;
            Cx:=X-Xmin;
            Ax:=#0C01;
            Int10 (Ax, Bx, Cx, Dx);
        end {if};
    end {Set_Point};



    (******************************************************************
    *                                                                 *
    *   procedure Del_Point                                           *
    *                                                                 *
    *******************************************************************
        *                                                             *
        *   Date        : 88-03-03                                    *
        *   Programmer  : M. Benz                                     *
        *                                                             *
        *   Purpose:                                                  *
        *       Lscht im Graphikmode einen Punkt auf dem Bildschirm. *
        *                                                             *
        *   In/Out:                                                   *
        *       procedure Del_Point (X, Y: integer)                   *
        *           X, Y : Koordinaten des Punktes auf dem Bildschirm.*
        *                                                             *
        **************************************************************)

    procedure Del_Point (X, Y: integer);
        var
            Ax, Bx, Cx, Dx      : word;
    begin
        if (X>=Xmin) and (X<=Xmax) and (Y>=Ymin) and 
           (Y<=Ymax) then begin
            Dx:=Ymax-Y;
            Cx:=X-Xmin;
            Ax:=#0C00;
            Int10 (Ax, Bx, Cx, Dx);
        end {if};
    end {Del_Point};



    (******************************************************************
    *                                                                 *
    *   procedure Line                                                *
    *                                                                 *
    *******************************************************************
        *                                                             *
        *   Date        : 88-03-03                                    *
        *   Programmer  : M. Benz                                     *
        *                                                             *
        *   Purpose:                                                  *
        *       Zeichnet eine Linie auf den Bildschirm.               *
        *                                                             *
        *   In/Out:                                                   *
        *       procedure Line (X1, Y1, X2, Y2: integer)              *
        *           X1, Y1 : Koordinaten des Anfangspunktes           *
        *           X2, Y2 : Koordinaten des Endpunktes               *
        *                                                             *
        **************************************************************)

    procedure Line (X1, Y1, X2, Y2: integer);
        var
            X, Y, SignX, SignY, I       : integer;
    begin
        if X1<X2 then SignX:=1
        else SignX:=-1;
        if Y1<Y2 then SignY:=1
        else SignY:=-1;
        if X1=X2 then begin
            for X:=0 to abs(Y1-Y2) do begin
                Y:=Y1+SignY*X;
                Set_Point (X1, Y);
            end {for};
        end {if}
        else begin
            if abs(X1-X2) > abs(Y1-Y2) then begin
                for X:=0 to abs(X1-X2) do begin
                    Y:=round(Y1+abs(Y1-Y2)/abs(X1-X2)*SignY*X);
                    Set_Point (X1+X*SignX, Y);
                end {for};
            end {if}
            else begin
                for Y:=0 to abs (Y1-Y2) do begin
                    X:=round(X1+abs(X1-X2)/abs(Y1-Y2)*SignX*Y);
                    Set_Point (X, Y1+Y*SignY);
                end {for};
            end {else};
        end {else};
    end {Line};



    (******************************************************************
    *                                                                 *
    *   procedure Mark_Point                                          *
    *                                                                 *
    *******************************************************************
        *                                                             *
        *   Date        : 88-03-03                                    *
        *   Programmer  : M. Benz                                     *
        *                                                             *
        *   Purpose:                                                  *
        *       Markiert einen Punkt auf dem Bildschirm.              *
        *                                                             *
        *   In/Out:                                                   *
        *       procedure Mark_Point (X, Y: integer)                  *
        *           X, Y : Koordinaten des zu markierenden Punktes.   *
        *                                                             *
        **************************************************************)

    procedure Mark_Point (X, Y: integer);
        var
            I               : integer;
    begin
        for I:=-2 to 2 do begin
            Set_Point (X+I, Y+I);
            Set_Point (X-I, Y+I);
        end {for};
    end {Mark_Point};



    (******************************************************************
    *                                                                 *
    *   function Read_Point                                           *
    *                                                                 *
    *******************************************************************
        *                                                             *
        *   Date        : 88-03-03                                    *
        *   Programmer  : M. Benz                                     *
        *                                                             *
        *   Purpose:                                                  *
        *       Gibt den Zustand (On/Off) eines Bildschirmpunktes     *
        *       zurck.                                               *
        *                                                             *
        *   In/Out:                                                   *
        *       function Read_Point (X, Y: integer): T_Status         *
        *           X, Y: Bildschirmkoordinaten des Punktes.          *
        *                                                             *
        **************************************************************)

    function Read_Point (X, Y: integer): T_Status;
        var
            Ax, Bx, Cx, Dx      : word;
    begin
        if (X>=Xmin) and (X<=Xmax) and (Y>=Ymin) and 
           (Y<=Ymax) then begin
            Dx:=Ymax-Y;
            Cx:=X-Xmin;
            Ax:=#0D00;
            Int10 (Ax, Bx, Cx, Dx);
            if Ax mod 2 = 0 then Read_Point:=Off
            else Read_Point:=On;
        end {if};
    end {Read_Point};

end {Graphic}.
