(******************************************************************************
*                                                                             *
*                                                                             *
*                       P    C    -    P  L  O  T  T  Y                       *
*                                                                             *
* IBMGC - TURBO                                                               *
*******************************************************************************
       *                                                                      *
       *  Version       : I 2.12                                              *
       *                                                                      *
       *  Date          : 21.12.87                                            *
       *                                                                      *
       *  Programmer    : J. Desteffani,  M. Benz                             *
       *                                                                      *
       *  Purpose:                                                            *
       *     Dieses Programm stellt eine Funktion graphisch auf dem Bild-     *
       *     schirm dar. Es wird dazu die IBM-Graphikkarte verwendet.         *
       *                                                                      *
       *  Input:                                                              *
       *     Explizite Eingabe der anzuzeigenden Funktion, X- und Y-Mass-     *
       *     stab, wobei der Y-Massstab auch automatisch gesetzt werden kann. *
       *                                                                      *
       *  Output:                                                             *
       *     Graphische Darstellung der eingegebenen Funktion auf dem Bild-   *
       *     schirm.                                                          *
       *                                                                      *
       *  Inout:                                                              *
       *     program PC_Plotty (input, output)                                *
       *     package Graphik                                                  *
       *     package Input                                                    *
       *     package Formel                                                   *
       *     procedure Help                                                   *
       *     procedure Plot                                                   *
       *                                                                      *
       ***********************************************************************)

program PC_Plotty (input, output);
    label
        10, 20, 30, 35, 40, 50;
    const
        Xmin = -30;
        Xmax = 609;
        Ymin = -14;
        Ymax = 186;
        Zmax =  25;
        Smax =  80;
        XtoY =   1.68;
        Ln10 =   2.3025851;
        MaxFun= 10;
    type
        T_Formel_Pointer = ^T_Formel;
        T_Formel         = record
                               Befehl      : integer;
                               Zahl        : real;
                               Left, Right : T_Formel_Pointer;
                           end;
        T_Root           = array [1..MaxFun] of T_Formel_Pointer;
        T_Register       = record
                               ax,bx,cx,dx,bp,si,di,ds,es,flags : integer;
                           end;
        T_Tabelle        = array [0..Xmax] of integer;
        T_Status         = (On, Off);
        T_String80       = string[80];
        T_StriArr        = array [1..MaxFun] of T_String80;
    var
        Stri                                 : T_String80;
        Formel                               : T_StriArr;
        Formel_File                          : text;
        I, Error, Yexp, NumFun, FunI         : integer;
        Xvon, Xbis, Yvon, Ybis, Yresol, Temp : real;
        Answer, Scale, Changed               : boolean;
        Ch                                   : char;
        Root                                 : T_Root;
        Val_Root                             : T_Formel_Pointer;


    {$I ibmgcpac.pas}     (* include package IBM-Graphik *)
    {$I input.pas}        (* include package Input       *)
    {$I formel.pas}       (* include package Formel      *)

    (***************************************************************************
    *                                                                          *
    *   procedure Help                                                         *
    *                                                                          *
    ****************************************************************************
          *                                                                    *
          *  Date        : 25.11.87                                            *
          *  Programmer  : M. Benz                                             *
          *                                                                    *
          *  Purpose:                                                          *
          *     Gibt einen Help-Text auf den Bildschirm.                       *
          *                                                                    *
          *  In/Out:                                                           *
          *     procedure Help                                                 *
          *                                                                    *
          *********************************************************************)

    procedure Help;
        var
            Helptxt               : text;
            I                     : integer;
            Cont                  : boolean;
            Ch                    : char;
            Stri                  : T_String80;
    begin
        Cont:=true;
        {$I-}
        assign (Helptxt, 'ibmplot.hlp');
        reset (Helptxt);
        if IOResult>0 then writeln ('Help-file not found')
        else begin
            clrscr;
            I:=1;
            while not eof (Helptxt) and Cont do begin
                readln (Helptxt, Stri);
                writeln (Stri);
                I:=I+1;
                if I=23 then begin
                    writeln; writeln;
                    write ('Press Q to quit or another key to continue ');
                    repeat until KeyPressed;
                    read (kbd,Ch);
                    if Ch in ['q','Q'] then Cont:=false;
                    clrscr;
                    I:=1;
                end;
            end;
            close (Helptxt);
            if Cont then begin
                writeln; writeln;
                write ('Press any key ');
                repeat until KeyPressed;
            end;
        end;
        {$I+}
    end;   (* Help *)



    (***************************************************************************
    *                                                                          *
    *   procedure Plot                                                         *
    *                                                                          *
    ****************************************************************************
          *                                                                    *
          *  Date        : 08.12.87                                            *
          *  Programmer  : M. Benz                                             *
          *                                                                    *
          *  Purpose:                                                          *
          *     Plottet Funktionen auf den Bildschirm.                         *
          *                                                                    *
          *  In/Out:                                                           *
          *     procedure Plot                                                 *
          *                                                                    *
          *********************************************************************)

    procedure Plot (NumFun: integer; Xvon, Xbis, Yvon, Ybis: real;
                    Root: T_Root; var Changed: boolean);
        var
            FunI, I         : integer;
            Tabelle         : T_Tabelle;
    begin
        Initgraph;
        if Scale then begin
            Skala (Xvon, Xbis, Yvon, Ybis);
        end;
        GotoXY (69,1);
        write ('working.....');
        for FunI:=1 to NumFun do begin
            if Changed then begin
                Tab_Calc (Root[FunI], Xvon, Xbis, Yvon, Ybis, Tabelle);
            end;
            if FunI=NumFun then begin
                GotoXY (69,1);
                write ('            ');
            end;
            for I:=0 to Xmax-1 do begin
                if (Tabelle[I]>-2) and (Tabelle[I+1]>-2) then begin
                    if ((Tabelle[I]>=0) and (Tabelle[I]<=Ymax)) or
                        ((Tabelle[I+1]>=0) and (Tabelle[I+1]<=Ymax))
                        then Line (I, Tabelle[I], I+1, Tabelle[I+1]);
                end;
            end;
        end; (* for *)
        repeat until KeyPressed;
        if NumFun=1 then Changed:=false;
        Inittext;
    end;   (* Plot *)


(******************************************************************************
*                                                                             *
*      M A I N                                                                *
*                                                                             *
*******************************************************************************
       *                                                                      *
       *  Date          : 21.12.87                                            *
       *                                                                      *
       *  Programmer    : M. Benz                                             *
       *                                                                      *
       ***********************************************************************)

begin
    ClrScr;
    writeln; writeln;
    writeln ('                       P   C   -   P   L   O   T   T   Y');
    writeln;
    writeln ('                                Version:  I 2.12');
    writeln;
    writeln ('                  Copyright 1987 by J. Desteffani and M. Benz');
    writeln; writeln;
    writeln ('This program plots a function on your screen');
    writeln ('It uses the IBM Graphics Card');
    writeln; writeln;
    writeln ('If you have an Hercules Graphics Card quit this program');
    writeln ('and use:  PLOTTY.COM');
    Initialisierung;
    NumFun:=1;
    for FunI:=1 to MaxFun do Formel[FunI]:='';
    FunI:=1;
    Error:=1;
    Xvon:=0; Xbis:=2*pi;
    Yvon:=-1; Ybis:=1;
    Scale:=true;
    Changed:=true;
    GotoXY (1,24);
    write ('Press any key ');
    repeat until KeyPressed;
    10:
    ClrScr;
    writeln ('                       P   C   -   P   L   O   T   T   Y');
    writeln; writeln;
    writeln ('N   Numbers of functions = ',NumFun);
    writeln;
    writeln ('F   Function ',FunI,': y(x)=',Formel[FunI]);
    writeln; writeln;
    writeln ('X   X-range  from ',Xvon:12,' to ',Xbis:12);
    writeln;
    writeln ('Y   Y-range  from ',Yvon:12,' to ',Ybis:12);
    writeln ('Q   Y-scale same as X-scale');
    writeln ('M   Y-range is min and max of the function');;
    writeln; writeln;
    writeln ('S   Scale on the screen = ',Scale,' ');
    writeln;
    writeln ('P   Plots the function on the screen');
    writeln;
    writeln ('H   Help in German');
    writeln;
    writeln ('E   Exit');
    writeln;
    write ('Press the desired key ');
    20:
    repeat until KeyPressed;
    read (kbd, Ch);
    case Ch of
        'N','n': begin
                    GotoXY (1,24); DelLine;
                    repeat
                        GotoXY (28,4); ClrEol;
                        NumFun:=trunc (Get_Real (NumFun));
                        GotoXY (1,24);
                        write ('Number out of range');
                    until (NumFun>0) and (NumFun<MaxFun);
                 end;  (* case Ch of N *)
        'H','h': begin
                    Help;
                    goto 10;
                 end;
        'F','f',^f:begin
                    30:
                    GotoXY (1,24);
                    if Ch=^f then write ('Please enter your filename')
                    else write ('Please enter your function');
                    for FunI:=1 to NumFun do begin
                        if Ch=^f then begin
                            GotoXY (14,6); ClrEol;
                            write (FunI,' from file ');
                            read (Stri);
                            assign (Formel_File, Stri); 
                            reset (Formel_File);
                            read (Formel_File, Stri);
                            close (Formel_File);
                            GotoXY (14,6); ClrEol;
                            write (FunI,': y(x)=',Stri);
                        end
                        else begin 
                            GotoXY (14,6); ClrEol;
                            write (FunI,': y(x)=');
                            35:
                            Stri:='';
                            read (Stri);
                        end; (* if Ch=^f *)
                        if Stri<>'' then Formel[FunI]:=Stri;
                        Translate_Formel (Formel[FunI], Root[FunI], Error);
                        GotoXY (22,6); ClrEol;
                        write (Formel[FunI]);
                        GotoXY (1,7); ClrEol;
                        if Error>0 then begin
                            GotoXY (21+Error,7);
                            write ('^ Unkown expression' );
                            GotoXY (22,6);
                            goto 35;
                        end;
                    end; (* for *)
                    Changed:=true;
                 end; (* case Ch of F *)
        'X','x': begin
                    40:
                    GotoXY (1,24);
                    write ('Please enter your range');
                    Stri:=''; I:=0;
                    repeat
                        GotoXY (19,9);
                        write ('             to ',Xbis:12);
                        GotoXY (19,9);
                        read (Stri);
                        if Stri<>'' then begin
                            Translate_Formel (Stri, Val_Root, I);
                            if I>0 then begin
                                GotoXY (1,24);
                                write ('ERROR in input          ');
                            end;
                            Changed:=true;
                        end;
                    until I=0;
                    if Stri<>'' then Calc (Val_Root, 0, Xvon, I);
                    GotoXY (19,9);
                    write (Xvon:12, ' to'); ClrEol;
                    Stri:=''; I:=0;
                    repeat
                        GotoXY (35,9); ClrEol;
                        read (Stri);
                        if Stri<>'' then begin
                            Translate_Formel (Stri, Val_Root, I);
                            if I>0 then begin
                                GotoXY (1,24);
                                write ('ERROR in input         ');
                            end;
                            Changed:=true;
                        end;
                    until I=0;
                    if Stri<>'' then Calc (Val_Root, 0, Xbis, I);
                    GotoXY (35,9);
                    write (Xbis:12); ClrEol;
                    if Xvon=Xbis then goto 40;
                    if Xvon>Xbis then begin
                        Temp:=Xvon;
                        Xvon:=Xbis;
                        Xbis:=Temp;
                    end;
                 end; (* case Ch of X *)
        'Q','q': begin
                    Yvon:=Xvon/XtoY;
                    Ybis:=Xbis/XtoY;
                    GotoXY (19,11);
                    write (Yvon:12);
                    GotoXY (35,11);
                    write (Ybis:12);
                    Changed:=true;
                 end; (* case Ch of Q *)
        'M','m': begin
                    if Error>0 then goto 30;
                    GotoXY (1,24);
                    DelLine;
                    write ('working.....');
                    Get_Y_Margins (Root[1], Xvon, Xbis, Yvon, Ybis);
                    if Ybis>0 then Yexp:=trunc(ln(Ybis)/Ln10)  (* log (Ybis) *)
                    else Yexp:=trunc(ln(-Yvon)/Ln10);
                    if Yexp<0 then Yexp:=Yexp-1;
                    Yresol:=exp((Yexp-3)*Ln10);               (* 10^(Yexp-3) *)
                    Yvon:=round (Yvon/Yresol-Yresol)*Yresol;
                    Ybis:=round (Ybis/Yresol+Yresol)*Yresol;
                    GotoXY (19,11);
                    write (Yvon:12);
                    GotoXY (35,11);
                    write (Ybis:12);
                    Changed:=true;
                 end; (* case Ch of M *)
        'Y','y': begin
                    50:
                    GotoXY (1,24);
                    write ('Please enter your range');
                    Stri:=''; I:=0;
                    repeat
                        GotoXY (19,11);
                        write ('             to ',Ybis:12); ClrEol;
                        GotoXY (19,11);
                        read (Stri);
                        if Stri<>'' then begin
                            Translate_Formel (Stri, Val_Root, I);
                            if I>0 then begin
                                GotoXY (1,24);
                                write ('ERROR in input          ');
                            end;
                            Changed:=true;
                        end;
                    until I=0;
                    if Stri<>'' then Calc (Val_Root, 0, Yvon, I);
                    GotoXY (19,11);
                    write (Yvon:12, ' to'); ClrEol;
                    Stri:=''; I:=0;
                    repeat
                        GotoXY (35,11); ClrEol;
                        read (Stri);
                        if Stri<>'' then begin
                            Translate_Formel (Stri, Val_Root, I);
                            if I>0 then begin
                                GotoXY (1,24);
                                write ('ERROR in input          ');
                            end;
                            Changed:=true;
                        end;
                    until I=0;
                    if Stri<>'' then Calc (Val_Root, 0, Ybis, I);
                    GotoXY (35,11);
                    write (Ybis:12); ClrEol;
                    if Yvon=Ybis then goto 50;
                    if Yvon>Ybis then begin
                        Temp:=Yvon;
                        Yvon:=Ybis;
                        Ybis:=Temp;
                    end;
                 end; (* case Ch of Y *)
        'S','s': begin
                    Scale:=not Scale;
                    GotoXY (27,16);
                    write (Scale,' ');
                 end;
        'P','p': begin
                    if (Error>0) or (FunI<NumFun) then goto 30;
                    Plot (NumFun, Xvon, Xbis, Yvon, Ybis, Root, Changed);
                    GotoXY (1,24);
                    DelLine;
                    goto 10;
                 end; (* case Ch of P *)
        else     ;
    end; (* case *)
    if not (Ch in ['E','e']) then begin
        GotoXY (1,24);
        DelLine;
        write ('Press the desired key: ');
        goto 20;
    end;

    GotoXY (1,24);
    write ('Do you want to quit PC-Plotty? ');
    Answer:=Get_Boolean;
    if not Answer then goto 10;
    ClrScr;
    GotoXY (25, 10);
    writeln ('Thank you and have a nice day');
    GotoXY (33, 14);
    writeln ('G O O D B Y E');
end.

