(**********************************************************************
*                                                                     *
*   package Skala                                                     *
*                                                                     *
***********************************************************************
    *                                                                 *
    *   Date:       88-04-07                                          *
    *   Programmer: M. Benz                                           *
    *                                                                 *
    *   Purpose:                                                      *
    *       Zeichnet Achsenkreuz und am Rand des Schirmes die Skala   *
    *                                                                 *
    *   In/Out:                                                       *
    *       procedure Str   (X: real; Exp: integer;                   *
    *                        var Stri: T_String20)                    *
    *                                                                 *
    ******************************************************************)

    const
        Xshf        =   10; (* Linksverschiebung der Skalenbeschr *)
        Yshf        =  -24; (* Hhenverschiebung der Skalenbeschr *)

    procedure Skala (Xvon, Xbis, Yvon, Ybis: real);
        var
            Xrange, Xresol, Xstep, Xcorr,
            Yrange, Yresol, Ystep, Ycorr, I, Ln10   : real;
            Xexp, Yexp, X0, Y0, Pos                 : integer;
            Stri, ExpS                              : T_String20;
            ExpB                                    : boolean;

        procedure Str (X: real; Exp: integer; var Stri: T_String20);
            var
                Chr0, Hund, Zehn, Ein   : integer;
        begin
            X:=abs(X)+0.001;
            Chr0:=ord('0');
            Stri:='';
            case Exp of
                2: begin
                    Hund:=trunc(X/100);
                    Zehn:=trunc((X-100*Hund)/10);
                    Ein :=trunc(X-100*Hund-10*Zehn);
                    Stri:=chr(Chr0+Hund)+chr(Chr0+Zehn);
                    Stri:=Stri+chr(Chr0+Ein);
                end {2};
                1: begin
                    Zehn:=trunc(X/10);
                    Ein :=trunc(X-10*Zehn);
                    Stri:=chr(Chr0+Zehn);
                    Stri:=Stri+chr(Chr0+Ein)+'.';
                end {1};
               -1: begin
                    Zehn:=trunc(10*X);
                    Stri:='.'+chr(Chr0+Zehn);
                    Stri:=Stri+chr(Chr0+trunc(100*(X-(0.1*Zehn))));
                end {-1};
                else begin
                    Ein :=trunc(X);
                    Stri:=chr(Chr0+Ein)+'.';
                    Stri:=Stri+chr(Chr0+trunc(10*(X-Ein)));
                end {else};
            end {case};
        end {Str};

    begin (* Skala *)
        Ln10:=ln(10);
        Xrange:=Xbis-Xvon; Yrange:=Ybis-Yvon;
        if Xbis>0 then Xexp:=trunc(ln(Xbis)/Ln10)     (* log (Xbis)  *)
        else Xexp:=trunc(ln(-Xvon)/Ln10);
        if Xexp<0 then Xexp:=Xexp-1;
        if Ybis>0 then Yexp:=trunc(ln(Ybis)/Ln10)     (* log (Ybis)  *)
        else Yexp:=trunc(ln(-Yvon)/Ln10);
        if Yexp<0 then Yexp:=Yexp-1;
        Xresol:=exp((Xexp-1)*Ln10);                   (* 10^(Xexp-1) *)
        Yresol:=exp((Yexp-1)*Ln10);                   (* 10^(Yexp-1) *)

        (* Schrittweite der Teilstriche berechnen *)
        Xstep:=Xrange/20; Ystep:=Yrange/20;
        if Xstep<=Xresol then Xstep:=Xresol
        else if Xstep<=2*Xresol then Xstep:=2*Xresol
        else if Xstep<=5*Xresol then Xstep:=5*Xresol
        else Xstep:=10*Xresol;
        if Ystep<Yresol then Ystep:=Yresol
        else if Ystep<=2*Yresol then Ystep:=2*Yresol
        else if Ystep<=5*Yresol then Ystep:=5*Yresol
        else Ystep:=10*Yresol;

        if Xrange>=Xresol then begin
            X0:=round(-Xvon/Xrange*(Xmax-Xmin)+Xmin);
        end {if}
        else X0:=-1;
        if Yrange>=Yresol then begin
            Y0:=Ymax-round(-Yvon/Yrange*(Ymax-Ymin)+Ymin);
        end {if}
        else Y0:=-1;
		(* Achsenkreuz *)
        if Y0>0 then Line (Xmin, Ymax-Ymin-Y0, Xmax, Ymax-Ymin-Y0);
        if X0>0 then Line (X0, Ymax-Ymin, X0, 0);
		(* Skalenrand *)
        Line (Xmin, Ymax-Ymin, Xmax, Ymax-Ymin);
        Line (Xmin, Ymax-Ymin, Xmin, 0);

        (* Teilstriche auf der X-Achse eintragen *)
        I:=0;
        Pos:=round (X0+(Xmax-Xmin)/Xrange*I);
        while Pos<=Xmax do begin
            if Pos>=Xmin then Line (Pos, Ymax-Ymin+5, Pos, Ymax-Ymin);
            I:=I+Xstep;
            Pos:=round (X0+(Xmax-Xmin)/Xrange*I);
        end {while};
        I:=-Xstep;
        Pos:=round (X0+(Xmax-Xmin)/Xrange*I);
        while Pos>=Xmin do begin
            Line (Pos, Ymax-Ymin+5, Pos, Ymax-Ymin);
            I:=I-Xstep;
            Pos:=round (X0+(Xmax-Xmin)/Xrange*I);
        end {while};

        (* Teilstriche auf der Y-Achse eintragen *)
        I:=0;
        Pos:=Ymax-round (Y0+(Ymax-Ymin)/Yrange*I);
        while Pos>=Ymin do begin
            if Pos<=Ymax then Line (Xmin-5, Pos, Xmin, Pos);
            I:=I+Ystep;
             Pos:=Ymax-round (Y0+(Ymax-Ymin)/Yrange*I);
        end {while};
        I:=-Ystep;
        Pos:=Ymax-round (Y0+(Ymax-Ymin)/Yrange*I);
        while Pos<=Ymax do begin
            Line (Xmin-5, Pos, Xmin, Pos);
            I:=I-Ystep;
            Pos:=Ymax-round (Y0+(Ymax-Ymin)/Yrange*I);
        end {while};

        (* Exponentenschreibweise auf X-Skala *)
        ExpB:=false;
        if (Xexp>2) or (Xexp<-1) then begin
            ExpB:=true;
            Str (Xexp, 0, ExpS);
            Xexp:=0;
            Xrange:=Xrange/10/Xresol;
            Xstep:=Xstep/10/Xresol;
        end {if};

        (* 0.0 und Vorzeichen auf X-Skala eintragen *)
        if (Xrange/Xstep<=12) then Xstep:=Xstep/2;
        if (Xrange/Xstep<=12) then Xstep:=Xstep/2;
        Xcorr:=(Xmax-Xmin)/Xrange;
(* ? *) if Xexp=2 then Xexp:=1;
        if X0>=Xmin then begin
            Str (0.0, Xexp, Stri);
            Pos:=X0-Xshf;
            OutTextXY (Pos, Ymax-8, Stri);
        end {if};
        if (X0>Xmin+20) and (X0<Xmax-20) then begin
            Pos:=round ((X0+Xcorr*2*Xstep)-Xshf);
            OutTextXY (Pos, Ymax-8, '+');
            Pos:=round ((X0-Xcorr*2*Xstep)-Xshf);
            OutTextXY (Pos, Ymax-8, '-');
        end {if};

        (* X-Skala beschriften *)
        I:=4*Xstep;
        Pos:=round ((X0+Xcorr*I)-Xshf);
        while Pos<=Xmax do begin
            Str (abs(I), Xexp, Stri);
            if Pos>=3 then begin
                OutTextXY (Pos, Ymax-8, Stri);
            end {if};
            I:=I+4*Xstep;
            Pos:=round ((X0+Xcorr*I)-Xshf);
        end {while};
        if ExpB and (X0<Xmax-20) then begin (* Exponent schreiben *)
            I:=I-6*Xstep;
            Pos:=round ((X0+Xcorr*I)-Xshf);
            Stri:='E';
            Stri:=Stri+ExpS;
            OutTextXY (Pos, Ymax-8, Stri);
        end {if};
        I:=-4*Xstep;
        Pos:=round ((X0+Xcorr*I)-Xshf);
        while Pos>=20 do begin
            Str (abs(I), Xexp, Stri);
            OutTextXY (Pos, Ymax-8, Stri);
            I:=I-4*Xstep;
            Pos:=round ((X0+Xcorr*I)-Xshf);
        end {while};
        if ExpB and (X0>=Xmax-20) then begin (* Exponent schreiben *)
            I:=I+6*Xstep;
            Pos:=round ((X0+Xcorr*I)-Xshf);
            Stri:='-E';
            Stri:=Stri+ExpS;
            OutTextXY (Pos, Ymax-8, Stri);
        end {if}
        else if X0>=Xmax-20 then begin
            I:=I+6*Xstep;
            Pos:=round ((X0+Xcorr*I)-Xshf);
            OutTextXY (Pos, Ymax-8, '-');
        end {else if};

        (* Exponentenschreibweise auf Y-Skala *)
        ExpB:=false;
        if (Yexp>2) or (Yexp<-1) then begin
            ExpB:=true;
            Str (Yexp, 0, ExpS);
            Yexp:=0;
            Yrange:=Yrange/10/Yresol;
            Ystep:=Ystep/10/Yresol;
        end {if};

        (* 0.0 und Vorzeichen auf Y-Skala eintragen *)
        if (Yrange/Ystep<=12) then Ystep:=Ystep/2;
        if (Yrange/Ystep<=12) then Ystep:=Ystep/2;
        Ycorr:=(Ymax-Ymin)/Yrange;
(* ? *) if Yexp=2 then Yexp:=1;
        if Y0>=Ymin then begin
            Str (0.0, Yexp, Stri);
            Pos:=Ymax-Y0+Yshf;
            OutTextXY (0, Pos, Stri);
        end {if};
        if (Y0>Ymin+20) and (Y0<Ymax-20) then begin
            Pos:=Ymax-round ((Y0+Ycorr*2*Ystep)-Yshf);
            OutTextXY (8, Pos, '+');
            Pos:=Ymax-round ((Y0-Ycorr*2*Ystep)-Yshf);
            OutTextXY (8, Pos, '-');
        end {if};

        (* Y-Skala beschriften *)
        I:=4*Ystep;
        Pos:=Ymax-round ((Y0+Ycorr*I)-Yshf);
        while Pos>=0 do begin
            Str (abs(I), Yexp, Stri);
            if Pos>=5 then begin
                OutTextXY (0, Pos, Stri);
            end {if};
            I:=I+4*Ystep;
            Pos:=Ymax-round ((Y0+Ycorr*I)-Yshf);
        end {while};
        if ExpB and (Y0<Ymax-20) then begin  (* Exponent schreiben *)
            I:=I-5*Ystep;
            Pos:=Ymax-round ((Y0+Ycorr*I)-Yshf);
            Stri:='E';
            Stri:=Stri+ExpS;
            OutTextXY (0, Pos, Stri);
        end {if};
        I:=-4*Ystep;
        Pos:=Ymax-round ((Y0+Ycorr*I)-Yshf);
        while Pos<=Ymax-5 do begin
            Str (abs(I), Yexp, Stri);
            OutTextXY (0, Pos, Stri);
            I:=I-4*Ystep;
            Pos:=Ymax-round ((Y0+Ycorr*I)-Yshf);
        end {while};
        if ExpB and (Y0>=Ymax-20) then begin (* Exponent schreiben *)
            I:=I+5*Ystep;
            Pos:=Ymax-round ((Y0+Ycorr*I)-Yshf);
            Stri:='E';
            Stri:=Stri+ExpS;
            OutTextXY (0, Pos, Stri);
            OutTextXY (8, Pos-8, '-');
        end {if}
        else if Y0>=Ymax-20 then begin
            I:=I+5*Ystep;
            Pos:=Ymax-round ((Y0+Ycorr*I)-Yshf);
            OutTextXY (8, Pos, '-');
        end {else if};

    end {Skala};
